#!/usr/bin/env python
# Copyright 2016 IBM Corp. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Example shows how to use the Adapter, Port, Virtual Switch, NIC, HBA
and Virtual Function interface.
"""

import sys
import yaml
import requests.packages.urllib3
import time

import zhmcclient

requests.packages.urllib3.disable_warnings()

if len(sys.argv) != 2:
    print("Usage: %s hmccreds.yaml" % sys.argv[0])
    sys.exit(2)
hmccreds_file = sys.argv[1]

with open(hmccreds_file, 'r') as fp:
    hmccreds = yaml.load(fp)

examples = hmccreds.get("examples", None)
if examples is None:
    print("examples not found in credentials file %s" % \
          (hmccreds_file))
    sys.exit(1)

adapter_port_vswitch = examples.get("adapter_port_vswitch", None)
if adapter_port_vswitch is None:
    print("adapter_port_vswitch not found in credentials file %s" % \
          (hmccreds_file))
    sys.exit(1)

loglevel = adapter_port_vswitch.get("loglevel", None)
if loglevel is not None:
    level = getattr(logging, loglevel.upper(), None)
    if level is None:
        print("Invalid value for loglevel in credentials file %s: %s" % \
              (hmccreds_file, loglevel))
        sys.exit(1)
    logging.basicConfig(level=level)

hmc = adapter_port_vswitch["hmc"]

cred = hmccreds.get(hmc, None)
if cred is None:
    print("Credentials for HMC %s not found in credentials file %s" % \
          (hmc, hmccreds_file))
    sys.exit(1)

userid = cred['userid']
password = cred['password']

print(__doc__)

print("Using HMC %s with userid %s ..." % (hmc, userid))
session = zhmcclient.Session(hmc, userid, password)
cl = zhmcclient.Client(session)

timestats = adapter_port_vswitch.get("timestats", None)
if timestats:
    session.time_stats_keeper.enable()

print("Listing CPCs ...")
cpcs = cl.cpcs.list()
for cpc in cpcs:
    print(cpc)
    print("\tListing Adapters for %s ..." % cpc.name)
    adapters = cpc.adapters.list()
    for i, adapter in enumerate(adapters):
        print('\t' + str(adapter))
        ports = adapter.ports.list(full_properties=False)
        for p, port in enumerate(ports):
            if p == 0:
                print("\t\tListing Ports for %s ..." % adapter.name)
#                port.pull_full_properties()
            print('\t\t' + str(port))
    print("\tListing Virtual Switches for %s ..." % cpc.name)
    vswitches = cpc.vswitches.list()
    for i, vswitch in enumerate(vswitches):
        print('\t' + str(vswitch))
    print("\tListing Partitions for %s ..." % cpc.name)
    partitions = cpc.partitions.list()
    for i, partition in enumerate(partitions):
        print('\t' + str(partition))
        nics = partition.nics.list(full_properties=False)
        for j, nic in enumerate(nics):
            if j == 0:
                print("\t\tListing NICs for %s ..." % partition.name)
            print('\t\t' + str(nic))

        hbas = partition.hbas.list(full_properties=False)
        for j, hba in enumerate(hbas):
            if j == 0:
                print("\t\tListing HBAs for %s ..." % partition.name)
            print('\t\t' + str(hba))
#                hba.pull_full_properties()
#                print('\t\t' + str(hba.properties))
        vfs = partition.virtual_functions.list(full_properties=False)
        for k, vf in enumerate(vfs):
            if k == 0:
                print("\t\tListing Virtual Functions for %s ..." % partition.name)
            print('\t\t' + str(vf))
#                vf.pull_full_properties()
#                print('\t\t' + str(vf.properties))

print("Logging off ...")
session.logoff()

if timestats:
    print(session.time_stats_keeper)

print("Done.")



#!/usr/bin/env python
# Copyright 2016 IBM Corp. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Unit tests for _nic module.
"""

from __future__ import absolute_import, print_function

import unittest
import requests_mock

from zhmcclient import Session, Client, Nic


class NicTests(unittest.TestCase):
    """All tests for Nic and NicManager classes."""

    def setUp(self):
        self.session = Session('test-dpm-host', 'test-user', 'test-id')
        self.client = Client(self.session)
        with requests_mock.mock() as m:
            # Because logon is deferred until needed, we perform it
            # explicitly in order to keep mocking in the actual test simple.
            m.post('/api/sessions', json={'api-session': 'test-session-id'})
            self.session.logon()

        self.cpc_mgr = self.client.cpcs
        with requests_mock.mock() as m:
            result = {
                'cpcs': [
                    {
                        'object-uri': '/api/cpcs/fake-cpc-id-1',
                        'name': 'CPC1',
                        'status': '',
                    }
                ]
            }
            m.get('/api/cpcs', json=result)
#            self.cpc = self.cpc_mgr.find(name="CPC1", full_properties=False)
            cpcs = self.cpc_mgr.list()
            self.cpc = cpcs[0]

        partition_mgr = self.cpc.partitions
        with requests_mock.mock() as m:
            result = {
                'partitions': [
                    {
                        'status': 'active',
                        'object-uri': '/api/partitions/fake-part-id-1',
                        'name': 'PART1'
                    },
                    {
                        'status': 'stopped',
                        'object-uri': '/api/partitions/fake-part-id-2',
                        'name': 'PART2'
                    }
                ]
            }

            m.get('/api/cpcs/fake-cpc-id-1/partitions', json=result)

            mock_result_part1 = {
                'status': 'active',
                'object-uri': '/api/partitions/fake-part-id-1',
                'name': 'PART1',
                'description': 'Test Partition',
                'more_properties': 'bliblablub',
                'nic-uris': [
                    '/api/partitions/fake-part-id-1/nics/fake-nic-id-1',
                    '/api/partitions/fake-part-id-1/nics/fake-nic-id-2'
                ]
            }
            m.get('/api/partitions/fake-part-id-1',
                  json=mock_result_part1)
            mock_result_part2 = {
                'status': 'stopped',
                'object-uri': '/api/partitions/fake-lpar-id-2',
                'name': 'PART2',
                'description': 'Test Partition',
                'more_properties': 'bliblablub',
                'nic-uris': [
                    '/api/partitions/fake-part-id-2/nics/fake-nic-id-4',
                    '/api/partitions/fake-part-id-2/nics/fake-nic-id-6'
                ]
            }
            m.get('/api/partitions/fake-part-id-2',
                  json=mock_result_part2)

            partitions = partition_mgr.list(full_properties=True)
            self.partition = partitions[0]

    def tearDown(self):
        with requests_mock.mock() as m:
            m.delete('/api/sessions/this-session', status_code=204)
            self.session.logoff()

    def test_init(self):
        """Test __init__() on NicManager instance in Partition."""
        nic_mgr = self.partition.nics
        self.assertEqual(nic_mgr.partition, self.partition)

    def test_list_short_ok(self):
        """
        Test successful list() with short set of properties on
        NicManager instance in partition.
        """
        nic_mgr = self.partition.nics
        nics = nic_mgr.list(full_properties=False)

        self.assertEqual(len(nics), len(self.partition.properties['nic-uris']))
        for idx, nic in enumerate(nics):
            self.assertEqual(
                nic.properties['element-uri'],
                self.partition.properties['nic-uris'][idx])
            self.assertEqual(
                nic.uri,
                self.partition.properties['nic-uris'][idx])
            self.assertFalse(nic.full_properties)
            self.assertEqual(nic.manager, nic_mgr)

    def test_list_full_ok(self):
        """
        Test successful list() with full set of properties on
        NicManager instance in partition.
        """
        nic_mgr = self.partition.nics

        with requests_mock.mock() as m:

            mock_result_nic1 = {
                'parent': '/api/partitions/fake-part-id-1',
                'name': 'nic1',
                'element-uri':
                    '/api/partitions/fake-part-id-1/nics/fake-nic-id-1',
                'class': 'nic',
                'element-id': 'fake-nic-id-1',
                'type': 'osd',
                'description': '',
                'more_properties': 'bliblablub'
            }
            m.get('/api/partitions/fake-part-id-1/nics/fake-nic-id-1',
                  json=mock_result_nic1)
            mock_result_nic2 = {
                'parent': '/api/partitions/fake-part-id-1',
                'name': 'nic2',
                'element-uri':
                    '/api/partitions/fake-part-id-1/nics/fake-nic-id-2',
                'class': 'nic',
                'element-id': 'fake-nic-id-2',
                'type': 'osd',
                'description': '',
                'more_properties': 'bliblablub'
            }
            m.get('/api/partitions/fake-part-id-1/nics/fake-nic-id-2',
                  json=mock_result_nic2)

            nics = nic_mgr.list(full_properties=True)

            self.assertEqual(
                len(nics),
                len(self.partition.properties['nic-uris']))
            for idx, nic in enumerate(nics):
                self.assertEqual(
                    nic.properties['element-uri'],
                    self.partition.properties['nic-uris'][idx])
                self.assertEqual(
                    nic.uri,
                    self.partition.properties['nic-uris'][idx])
                self.assertTrue(nic.full_properties)
                self.assertEqual(nic.manager, nic_mgr)

    def test_create(self):
        """
        This tests the 'Create NIC' operation.
        """
        nic_mgr = self.partition.nics
        with requests_mock.mock() as m:
            result = {
                'element-uri':
                    '/api/partitions/fake-part-id-1/nics/fake-nic-id-1'
            }
            m.post('/api/partitions/fake-part-id-1/nics', json=result)

            nic = nic_mgr.create(properties={})

            self.assertTrue(isinstance(nic, Nic))
            self.assertEqual(nic.properties, result)
            self.assertEqual(nic.uri, result['element-uri'])

    def test_delete(self):
        """
        This tests the 'Delete NIC' operation.
        """
        nic_mgr = self.partition.nics
        nics = nic_mgr.list(full_properties=False)
        nic = nics[0]
        with requests_mock.mock() as m:
            result = {}
            m.delete(
                '/api/partitions/fake-part-id-1/nics/fake-nic-id-1',
                json=result)
            status = nic.delete()
            self.assertEqual(status, None)

    def test_update_properties(self):
        """
        This tests the 'Update NIC Properties' operation.
        """
        nic_mgr = self.partition.nics
        nics = nic_mgr.list(full_properties=False)
        nic = nics[0]
        with requests_mock.mock() as m:
            result = {}
            m.post(
                '/api/partitions/fake-part-id-1/nics/fake-nic-id-1',
                json=result)
            status = nic.update_properties(properties={})
            self.assertEqual(status, None)

    def test_nic_object(self):
        """
        This tests the `nic_object()` method.
        """
        nic_mgr = self.partition.nics
        nic_id = 'fake-nic-id0711'

        nic = nic_mgr.nic_object(nic_id)

        nic_uri = self.partition.uri + "/nics/" + nic_id

        self.assertTrue(isinstance(nic, Nic))
        self.assertEqual(nic.uri, nic_uri)
        self.assertEqual(nic.properties['element-uri'], nic_uri)
        self.assertEqual(nic.properties['element-id'], nic_id)
        self.assertEqual(nic.properties['class'], 'nic')
        self.assertEqual(nic.properties['parent'], self.partition.uri)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# Copyright 2016 IBM Corp. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Unit tests for _hba module.
"""

from __future__ import absolute_import, print_function

import unittest
import requests_mock

from zhmcclient import Session, Client, Hba, Adapter, Port


class HbaTests(unittest.TestCase):
    """All tests for Hba and HbaManager classes."""

    def setUp(self):
        self.session = Session('test-dpm-host', 'test-user', 'test-id')
        self.client = Client(self.session)
        with requests_mock.mock() as m:
            # Because logon is deferred until needed, we perform it
            # explicitly in order to keep mocking in the actual test simple.
            m.post('/api/sessions', json={'api-session': 'test-session-id'})
            self.session.logon()

        self.cpc_mgr = self.client.cpcs
        with requests_mock.mock() as m:
            result = {
                'cpcs': [
                    {
                        'object-uri': '/api/cpcs/fake-cpc-id-1',
                        'name': 'CPC1',
                        'status': '',
                    }
                ]
            }
            m.get('/api/cpcs', json=result)
#            self.cpc = self.cpc_mgr.find(name="CPC1", full_properties=False)
            cpcs = self.cpc_mgr.list()
            self.cpc = cpcs[0]

        partition_mgr = self.cpc.partitions
        with requests_mock.mock() as m:
            result = {
                'partitions': [
                    {
                        'status': 'active',
                        'object-uri': '/api/partitions/fake-part-id-1',
                        'name': 'PART1'
                    },
                    {
                        'status': 'stopped',
                        'object-uri': '/api/partitions/fake-part-id-2',
                        'name': 'PART2'
                    }
                ]
            }

            m.get('/api/cpcs/fake-cpc-id-1/partitions', json=result)

            mock_result_part1 = {
                'status': 'active',
                'object-uri': '/api/partitions/fake-part-id-1',
                'name': 'PART1',
                'description': 'Test Partition',
                'more_properties': 'bliblablub',
                'hba-uris': [
                    '/api/partitions/fake-part-id-1/hbas/fake-hba-id-1',
                    '/api/partitions/fake-part-id-1/hbas/fake-hba-id-2'
                ]
            }
            m.get('/api/partitions/fake-part-id-1',
                  json=mock_result_part1)
            mock_result_part2 = {
                'status': 'stopped',
                'object-uri': '/api/partitions/fake-lpar-id-2',
                'name': 'PART2',
                'description': 'Test Partition',
                'more_properties': 'bliblablub',
                'hba-uris': [
                    '/api/partitions/fake-part-id-2/hbas/fake-hba-id-4',
                    '/api/partitions/fake-part-id-2/hbas/fake-hba-id-6'
                ]
            }
            m.get('/api/partitions/fake-part-id-2',
                  json=mock_result_part2)

            partitions = partition_mgr.list(full_properties=True)
            self.partition = partitions[0]

    def tearDown(self):
        with requests_mock.mock() as m:
            m.delete('/api/sessions/this-session', status_code=204)
            self.session.logoff()

    def test_init(self):
        """Test __init__() on HbaManager instance in Partition."""
        hba_mgr = self.partition.hbas
        self.assertEqual(hba_mgr.partition, self.partition)

    def test_list_short_ok(self):
        """
        Test successful list() with short set of properties on
        HbaManager instance in partition.
        """
        hba_mgr = self.partition.hbas
        hbas = hba_mgr.list(full_properties=False)

        self.assertEqual(len(hbas), len(self.partition.properties['hba-uris']))
        for idx, hba in enumerate(hbas):
            self.assertEqual(
                hba.properties['element-uri'],
                self.partition.properties['hba-uris'][idx])
            self.assertEqual(
                hba.uri,
                self.partition.properties['hba-uris'][idx])
            self.assertFalse(hba.full_properties)
            self.assertEqual(hba.manager, hba_mgr)

    def test_list_full_ok(self):
        """
        Test successful list() with full set of properties on
        HbaManager instance in partition.
        """
        hba_mgr = self.partition.hbas

        with requests_mock.mock() as m:

            mock_result_hba1 = {
                'parent': '/api/partitions/fake-part-id-1',
                'name': 'hba1',
                'element-uri':
                    '/api/partitions/fake-part-id-1/hbas/fake-hba-id-1',
                'class': 'hba',
                'element-id': 'fake-hba-id-1',
                'wwpn': 'AABBCCDDEC000082',
                'description': '',
                'more_properties': 'bliblablub'
            }
            m.get('/api/partitions/fake-part-id-1/hbas/fake-hba-id-1',
                  json=mock_result_hba1)
            mock_result_hba2 = {
                'parent': '/api/partitions/fake-part-id-1',
                'name': 'hba2',
                'element-uri':
                    '/api/partitions/fake-part-id-1/hbas/fake-hba-id-2',
                'class': 'hba',
                'element-id': 'fake-hba-id-2',
                'wwpn': 'AABBCCDDEC000083',
                'description': '',
                'more_properties': 'bliblablub'
            }
            m.get('/api/partitions/fake-part-id-1/hbas/fake-hba-id-2',
                  json=mock_result_hba2)

            hbas = hba_mgr.list(full_properties=True)

            self.assertEqual(
                len(hbas),
                len(self.partition.properties['hba-uris']))
            for idx, hba in enumerate(hbas):
                self.assertEqual(
                    hba.properties['element-uri'],
                    self.partition.properties['hba-uris'][idx])
                self.assertEqual(
                    hba.uri,
                    self.partition.properties['hba-uris'][idx])
                self.assertTrue(hba.full_properties)
                self.assertEqual(hba.manager, hba_mgr)

    def test_create(self):
        """
        This tests the 'Create HBA' operation.
        """
        hba_mgr = self.partition.hbas
        with requests_mock.mock() as m:
            result = {
                'element-uri':
                    '/api/partitions/fake-part-id-1/hbas/fake-hba-id-1'
            }
            m.post('/api/partitions/fake-part-id-1/hbas', json=result)

            hba = hba_mgr.create(properties={})

            self.assertTrue(isinstance(hba, Hba))
            self.assertEqual(hba.properties, result)
            self.assertEqual(hba.uri, result['element-uri'])

    def test_delete(self):
        """
        This tests the 'Delete HBA' operation.
        """
        hba_mgr = self.partition.hbas
        hbas = hba_mgr.list(full_properties=False)
        hba = hbas[0]
        with requests_mock.mock() as m:
            result = {}
            m.delete(
                '/api/partitions/fake-part-id-1/hbas/fake-hba-id-1',
                json=result)
            status = hba.delete()
            self.assertEqual(status, None)

    def test_update_properties(self):
        """
        This tests the 'Update HBA Properties' operation.
        """
        hba_mgr = self.partition.hbas
        hbas = hba_mgr.list(full_properties=False)
        hba = hbas[0]
        with requests_mock.mock() as m:
            result = {}
            m.post(
                '/api/partitions/fake-part-id-1/hbas/fake-hba-id-1',
                json=result)
            status = hba.update_properties(properties={})
            self.assertEqual(status, None)

    def test_reassign_port(self):
        """
        This tests the 'reassign_port()' method.
        """
        hba_mgr = self.partition.hbas
        hba_uri = '/api/partitions/fake-part-id-1/hbas/fake-hba-id-1'
        hba = Hba(hba_mgr, hba_uri)

        adapter_mgr = self.cpc.adapters
        adapter_uri = '/api/adapters/fake-adapter-id-1'
        adapter = Adapter(adapter_mgr, adapter_uri)

        port_mgr = adapter.ports
        port2_uri = '/api/adapters/fake-adapter-id-2/'\
                    'storage-ports/fake-port-id-2'
        port2 = Port(port_mgr, port2_uri)

        with requests_mock.mock() as m:
            # TODO: Add the request body to the mock call:
            # request_reassign = {
            #     'adapter-port-uri': port2_uri
            # }
            m.post(hba_uri + '/operations/reassign-storage-adapter-port',
                   json={})

            hba.reassign_port(port2)


if __name__ == '__main__':
    unittest.main()



# Copyright 2017 IBM Corp. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Public constants.

These constants are not meant to be changed by the user, they are made
available for inspection and documentation purposes only.

For technical reasons, the online documentation shows these constants in the
``zhmcclient._constants`` namespace, but they are also available in the
``zhmcclient`` namespace and should be used from there.
"""

__all__ = ['DEFAULT_CONNECT_TIMEOUT',
           'DEFAULT_CONNECT_RETRIES',
           'DEFAULT_READ_TIMEOUT',
           'DEFAULT_READ_RETRIES',
           'DEFAULT_MAX_REDIRECTS',
           'DEFAULT_OPERATION_TIMEOUT',
           'DEFAULT_STATUS_TIMEOUT',
           'HMC_LOGGER_NAME',
           'API_LOGGER_NAME']


#: Default HTTP connect timeout in seconds,
#: if not specified in the ``retry_timeout_config`` init argument to
#: :class:`~zhmcclient.Session`.
DEFAULT_CONNECT_TIMEOUT = 30

#: Default number of HTTP connect retries,
#: if not specified in the ``retry_timeout_config`` init argument to
#: :class:`~zhmcclient.Session`.
DEFAULT_CONNECT_RETRIES = 3

#: Default HTTP read timeout in seconds,
#: if not specified in the ``retry_timeout_config`` init argument to
#: :class:`~zhmcclient.Session`.
DEFAULT_READ_TIMEOUT = 300

#: Default number of HTTP read retries,
#: if not specified in the ``retry_timeout_config`` init argument to
#: :class:`~zhmcclient.Session`.
DEFAULT_READ_RETRIES = 3

#: Default max. number of HTTP redirects,
#: if not specified in the ``retry_timeout_config`` init argument to
#: :class:`~zhmcclient.Session`.
DEFAULT_MAX_REDIRECTS = 30

#: Default timeout in seconds for waiting for completion of an asynchronous
#: HMC operation. This is used as a default value in asynchronous methods on
#: resource objects (e.g. :meth:`zhmcclient.Partition.start`), in the
#: :meth:`zhmcclient.Job.wait_for_completion` method, and in the
#: low level method :meth:`zhmcclient.Session.post`.
DEFAULT_OPERATION_TIMEOUT = 3600

#: Default timeout in seconds for waiting for completion of deferred status
#: changes for LPARs. This is used as a default value in asynchronous methods
#: of the :class:`~zhmcclient.Lpar` class that change its status (e.g.
#: :meth:`zhmcclient.Lpar.activate`)).
DEFAULT_STATUS_TIMEOUT = 60

#: Name of the Python logger that logs HMC operations.
HMC_LOGGER_NAME = 'zhmcclient.hmc'

#: Name of the Python logger that logs zhmcclient API calls made by the user.
API_LOGGER_NAME = 'zhmcclient.api'



#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import absolute_import

import os
import sys
import requests.packages.urllib3
import click
import click_repl
from prompt_toolkit.history import FileHistory
import logging
from logging.handlers import SysLogHandler
from logging import StreamHandler
import platform

from ._helper import CmdContext, GENERAL_OPTIONS_METAVAR, REPL_HISTORY_FILE, \
    REPL_PROMPT, TABLE_FORMATS, LOG_LEVELS, LOG_COMPONENTS, LOG_DESTINATIONS, \
    SYSLOG_FACILITIES

requests.packages.urllib3.disable_warnings()


# Default values for some options
DEFAULT_OUTPUT_FORMAT = 'table'
DEFAULT_TIMESTATS = False
DEFAULT_LOG = 'all=warning'
DEFAULT_LOG_DESTINATION = 'stderr'
DEFAULT_SYSLOG_FACILITY = 'user'

SYSLOG_ADDRESSES = {
    'Linux': '/dev/log',
    'Darwin': '/var/run/syslog',  # OS-X
    'Windows': ('localhost', 514),
}


@click.group(invoke_without_command=True,
             options_metavar=GENERAL_OPTIONS_METAVAR)
@click.option('-h', '--host', type=str, envvar='ZHMC_HOST',
              help="Hostname or IP address of the HMC "
                   "(Default: ZHMC_HOST environment variable).")
@click.option('-u', '--userid', type=str, envvar='ZHMC_USERID',
              help="Username for the HMC "
                   "(Default: ZHMC_USERID environment variable).")
@click.option('-p', '--password', type=str, envvar='ZHMC_PASSWORD',
              help="Password for the HMC "
                   "(Default: ZHMC_PASSWORD environment variable).")
@click.option('-o', '--output-format', type=click.Choice(TABLE_FORMATS +
              ['json']),
              help='Output format (Default: {of}).'
              .format(of=DEFAULT_OUTPUT_FORMAT))
@click.option('-t', '--timestats', type=str, is_flag=True,
              help='Show time statistics of HMC operations.')
@click.option('--log', type=str, metavar='COMP=LEVEL,...',
              help="Set a component to a log level (COMP: [{c}], "
              "LEVEL: [{l}], Default: {d})."
              .format(c='|'.join(LOG_COMPONENTS),
                      l='|'.join(LOG_LEVELS),
                      d=DEFAULT_LOG))
@click.option('--log-dest', type=click.Choice(LOG_DESTINATIONS),
              help="Log destination for this command (Default: {ld})."
              .format(ld=DEFAULT_LOG_DESTINATION))
@click.option('--syslog-facility', type=click.Choice(SYSLOG_FACILITIES),
              help="Syslog facility when logging to the syslog "
              "(Default: {slf})."
              .format(slf=DEFAULT_SYSLOG_FACILITY))
@click.version_option(help="Show the version of this command and exit.")
@click.pass_context
def cli(ctx, host, userid, password, output_format, timestats, log, log_dest,
        syslog_facility):
    """
    Command line interface for the z Systems HMC.

    The options shown in this help text are general options that can also
    be specified on any of the (sub-)commands.
    """

    # Concept: In interactive mode, the global options specified in the command
    # line are used as defaults for the commands that are issued interactively.
    # The interactive commands may override these options.
    # This requires being able to determine for each option whether it has been
    # specified. This is the reason the options don't define defaults in the
    # decorators that define them.

    if ctx.obj is None:
        # We are in command mode or are processing the command line options in
        # interactive mode.
        # We apply the documented option defaults.
        if output_format is None:
            output_format = DEFAULT_OUTPUT_FORMAT
        if timestats is None:
            timestats = DEFAULT_TIMESTATS
    else:
        # We are processing an interactive command.
        # We apply the option defaults from the command line options.
        if host is None:
            host = ctx.obj.host
        if userid is None:
            userid = ctx.obj.userid
        if password is None:
            password = ctx.obj._password
        if output_format is None:
            output_format = ctx.obj.output_format
        if timestats is None:
            timestats = ctx.obj.timestats

    # TODO: Add context support for the following options:
    if log is None:
        log = DEFAULT_LOG
    if log_dest is None:
        log_dest = DEFAULT_LOG_DESTINATION
    if syslog_facility is None:
        syslog_facility = DEFAULT_SYSLOG_FACILITY

    # Now we have the effective values for the options as they should be used
    # by the current command, regardless of the mode.

    # Set up logging
    if log_dest == 'syslog':
        # The choices in SYSLOG_FACILITIES have been validated by click
        # so we don't need to further check them.
        facility = SysLogHandler.facility_names[syslog_facility]
        system = platform.system()
        try:
            address = SYSLOG_ADDRESSES[system]
        except KeyError:
            raise NotImplementedError(
                "Logging to syslog is not supported on this platform: {}".
                format(system))
        handler = SysLogHandler(address=address, facility=facility)
        format_string = '%(levelname)s %(name)s: %(message)s'
    elif log_dest == 'stderr':
        handler = StreamHandler(stream=sys.stderr)
        format_string = '%(levelname)s %(name)s: %(message)s'
    else:
        # The choices in LOG_DESTINATIONS have been validated by click
        assert log_dest == 'none'
        handler = None
        format_string = None

    log_specs = log.split(',')
    for log_spec in log_specs:

        # ignore extra ',' at begin, end or in between
        if log_spec == '':
            continue

        try:
            log_comp, log_level = log_spec.split('=', 1)
        except ValueError:
            raise click.ClickException(
                "Missing '=' "
                "in COMP=LEVEL specification in --log option: {ls}"
                .format(ls=log_spec))

        level = getattr(logging, log_level.upper(), None)
        if level is None:
            raise click.ClickException(
                "Invalid log level "
                "in COMP=LEVEL specification in --log option: {ls}"
                .format(ls=log_spec))

        if log_comp not in LOG_COMPONENTS:
            raise click.ClickException(
                "Invalid log component "
                "in COMP=LEVEL specification in --log option: {ls}"
                .format(ls=log_spec))

        if handler:
            handler.setFormatter(logging.Formatter(format_string))
            if log_comp == 'all':
                logger = logging.getLogger('')
                logger.addHandler(handler)
                logger.setLevel(level)
            else:
                if log_comp == 'api':
                    logger = logging.getLogger('zhmcclient.api')
                    logger.addHandler(handler)
                    logger.setLevel(level)
                if log_comp == 'hmc':
                    logger = logging.getLogger('zhmcclient.hmc')
                    logger.addHandler(handler)
                    logger.setLevel(level)

    session_id = os.environ.get('ZHMC_SESSION_ID', None)

    def get_password_via_prompt(host, userid):
        """
        Password retrieval function that prompts for the password.

        It follows the interface defined in
        :func:`~zhmcclient.get_password_interface` and needs access to the
        click context (ctx).
        """
        if userid is not None and host is not None:
            ctx.obj.spinner.stop()
            password = click.prompt(
                "Enter password (for user {userid} at HMC {host})"
                .format(userid=userid, host=host), hide_input=True,
                confirmation_prompt=False, type=str, err=True)
            ctx.obj.spinner.start()
            return password
        else:
            raise click.ClickException("{cmd} command requires logon, but "
                                       "no session-id or userid provided."
                                       .format(cmd=ctx.invoked_subcommand))

    # We create a command context for each command: An interactive command has
    # its own command context different from the command context for the
    # command line.
    ctx.obj = CmdContext(host, userid, password, output_format, timestats,
                         session_id, get_password_via_prompt)

    # Invoke default command
    if ctx.invoked_subcommand is None:
        ctx.invoke(repl)


@cli.command('help')
@click.pass_context
def repl_help(ctx):
    """
    Show help message for interactive mode.
    """
    print("""
The following can be entered in interactive mode:

  <zhmc-cmd>                  Execute zhmc command <zhmc-cmd>.
  !<shell-cmd>                Execute shell command <shell-cmd>.

  <CTRL-D>, :q, :quit, :exit  Exit interactive mode.

  <TAB>                       Tab completion (can be used anywhere).
  --help                      Show zhmc general help message, including a list
                              of zhmc commands.
  <zhmc-cmd> --help           Show help message for zhmc command <zhmc-cmd>.
  help                        Show this help message.
  :?, :h, :help               Show (incomplete) help message about interactive
                              mode.
""")


@cli.command('repl')
@click.pass_context
def repl(ctx):
    """
    Enter interactive (REPL) mode (default).
    """

    history_file = REPL_HISTORY_FILE
    if history_file.startswith('~'):
        history_file = os.path.expanduser(history_file)

    print("Enter 'help' for help, <CTRL-D> or ':q' to exit.")

    prompt_kwargs = {
        'message': REPL_PROMPT,
        'history': FileHistory(history_file),
    }
    click_repl.repl(ctx, prompt_kwargs=prompt_kwargs)


# TODO: Apparently registering is not needed, clarify that.
# click_repl.register_repl(repl)



#!/usr/bin/env python
# Copyright 2016 IBM Corp. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Unit tests for _client module.
"""

from __future__ import absolute_import, print_function

import unittest
import requests_mock

from zhmcclient import Session, Client


class ClientTests(unittest.TestCase):
    """All tests for Client classes."""

    def setUp(self):
        self.session = Session('fake-host')
        self.client = Client(self.session)
        with requests_mock.mock() as m:
            # Because logon is deferred until needed, we perform it
            # explicitly in order to keep mocking in the actual test simple.
            m.post('/api/sessions', json={'api-session': 'fake-session-id'})

    def tearDown(self):
        with requests_mock.mock() as m:
            m.delete('/api/sessions/this-session', status_code=204)

    def test_init(self):
        """Test Initialization of Client class."""
        self.assertTrue(self.client.session is self.session)

    def test_version_info_ok(self):
        """
        Test successful version_info().
        """
        with requests_mock.mock() as m:
            result = {
                'hmc-name': 'FAKEHMC1',
                'hmc-version': '2.13.1',
                'api-minor-version': 7,
                'api-features': ['internal-get-files-from-se'],
                'api-major-version': 1
            }
            m.get('/api/version', json=result)

            vi = self.client.version_info()
            self.assertEqual(vi[0], result['api-major-version'])
            self.assertEqual(vi[1], result['api-minor-version'])

    def test_query_api_version_ok(self):
        """
        Test successful query_api_version().
        """
        with requests_mock.mock() as m:
            result = {
                'hmc-name': 'FAKEHMC1',
                'hmc-version': '2.13.1',
                'api-minor-version': 7,
                'api-features': ['internal-get-files-from-se'],
                'api-major-version': 1
            }
            m.get('/api/version', json=result)

            api_version = self.client.query_api_version()
            vi = self.client.version_info()
            self.assertEqual(vi[0], api_version['api-major-version'])
            self.assertEqual(vi[1], api_version['api-minor-version'])


if __name__ == '__main__':
    unittest.main()



# Copyright 2016 IBM Corp. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
A :term:`LPAR` (Logical Partition) is a subset of the hardware resources of a
:term:`CPC` in classic mode (or ensemble mode), virtualized as a separate
computer.

LPARs cannot be created or deleted by the user; they can only be listed.

LPAR resources are contained in CPC resources.

LPAR resources only exist in CPCs that are in classic mode (or ensemble mode).
CPCs in DPM mode have :term:`Partition` resources, instead.
"""

from __future__ import absolute_import

import time

from ._manager import BaseManager
from ._resource import BaseResource
from ._exceptions import StatusTimeout
from ._logging import get_logger, logged_api_call

__all__ = ['LparManager', 'Lpar']

LOG = get_logger(__name__)


class LparManager(BaseManager):
    """
    Manager providing access to the :term:`LPARs <LPAR>` in a particular
    :term:`CPC`.

    Derived from :class:`~zhmcclient.BaseManager`; see there for common methods
    and attributes.

    Objects of this class are not directly created by the user; they are
    accessible via the following instance variable of a
    :class:`~zhmcclient.Cpc` object (in DPM mode):

    * :attr:`~zhmcclient.Cpc.lpars`
    """

    def __init__(self, cpc):
        # This function should not go into the docs.
        # Parameters:
        #   cpc (:class:`~zhmcclient.Cpc`):
        #     CPC defining the scope for this manager.

        # Resource properties that are supported as filter query parameters.
        # If the support for a resource property changes within the set of HMC
        # versions that support this type of resource, this list must be set up
        # for the version of the HMC this session is connected to.
        query_props = [
            'name',
        ]

        super(LparManager, self).__init__(
            resource_class=Lpar,
            parent=cpc,
            uri_prop='object-uri',
            name_prop='name',
            query_props=query_props)

    @property
    def cpc(self):
        """
        :class:`~zhmcclient.Cpc`: :term:`CPC` defining the scope for this
        manager.
        """
        return self._parent

    @logged_api_call
    def list(self, full_properties=False, filter_args=None):
        """
        List the LPARs in this CPC.

        Authorization requirements:

        * Object-access permission to this CPC.
        * Object-access permission to any LPAR to be included in the result.

        Parameters:

          full_properties (bool):
            Controls whether the full set of resource properties should be
            retrieved, vs. only the short set as returned by the list
            operation.

          filter_args (dict):
            Filter arguments that narrow the list of returned resources to
            those that match the specified filter arguments. For details, see
            :ref:`Filtering`.

            `None` causes no filtering to happen, i.e. all resources are
            returned.

        Returns:

          : A list of :class:`~zhmcclient.Lpar` objects.

        Raises:

          :exc:`~zhmcclient.HTTPError`
          :exc:`~zhmcclient.ParseError`
          :exc:`~zhmcclient.AuthError`
          :exc:`~zhmcclient.ConnectionError`
        """
        query_parms, client_filters = self._divide_filter_args(filter_args)

        resources_name = 'logical-partitions'
        uri = '{}/{}{}'.format(self.cpc.uri, resources_name, query_parms)

        resource_obj_list = []
        result = self.session.get(uri)
        if result:
            props_list = result[resources_name]
            for props in props_list:

                resource_obj = self.resource_class(
                    manager=self,
                    uri=props[self._uri_prop],
                    name=props.get(self._name_prop, None),
                    properties=props)

                if self._matches_filters(resource_obj, client_filters):
                    resource_obj_list.append(resource_obj)
                    if full_properties:
                        resource_obj.pull_full_properties()

        return resource_obj_list


class Lpar(BaseResource):
    """
    Representation of an :term:`LPAR`.

    Derived from :class:`~zhmcclient.BaseResource`; see there for common
    methods and attributes.

    Objects of this class are not directly created by the user; they are
    returned from creation or list functions on their manager object
    (in this case, :class:`~zhmcclient.LparManager`).
    """

    def __init__(self, manager, uri, name=None, properties=None):
        # This function should not go into the docs.
        #   manager (:class:`~zhmcclient.LparManager`):
        #     Manager object for this resource object.
        #   uri (string):
        #     Canonical URI path of the resource.
        #   name (string):
        #     Name of the resource.
        #   properties (dict):
        #     Properties to be set for this resource object. May be `None` or
        #     empty.
        if not isinstance(manager, LparManager):
            raise AssertionError("Lpar init: Expected manager type %s, "
                                 "got %s" %
                                 (LparManager, type(manager)))
        super(Lpar, self).__init__(manager, uri, name, properties)

    @logged_api_call
    def activate(self, wait_for_completion=True,
                 operation_timeout=None, status_timeout=None):
        """
        Activate (start) this LPAR, using the HMC operation "Activate Logical
        Partition".

        This HMC operation has deferred status behavior: If the asynchronous
        job on the HMC is complete, it takes a few seconds until the LPAR
        status has reached the desired value. If `wait_for_completion=True`,
        this method repeatedly checks the status of the LPAR after the HMC
        operation has completed, and waits until the status is in the desired
        state "not-operating" (which indicates that the LPAR is active but
        no operating system is running).

        Authorization requirements:

        * Object-access permission to the CPC containing this LPAR.
        * Object-access permission to this LPAR.
        * Task permission for the "Activate" task.

        Parameters:

          wait_for_completion (bool):
            Boolean controlling whether this method should wait for completion
            of the requested asynchronous HMC operation, as follows:

            * If `True`, this method will wait for completion of the
              asynchronous job performing the operation, and for the status
              becoming "not-operating".

            * If `False`, this method will return immediately once the HMC has
              accepted the request to perform the operation.

          operation_timeout (:term:`number`):
            Timeout in seconds, for waiting for completion of the asynchronous
            job performing the operation. The special value 0 means that no
            timeout is set. `None` means that the default async operation
            timeout of the session is used. If the timeout expires when
            `wait_for_completion=True`, a
            :exc:`~zhmcclient.OperationTimeout` is raised.

          status_timeout (:term:`number`):
            Timeout in seconds, for waiting that the status of the LPAR has
            reached the desired status, after the HMC operation has completed.
            The special value 0 means that no timeout is set. `None` means that
            the default async operation timeout of the session is used.
            If the timeout expires when `wait_for_completion=True`, a
            :exc:`~zhmcclient.StatusTimeout` is raised.

        Returns:

          `None` or :class:`~zhmcclient.Job`:

            If `wait_for_completion` is `True`, returns `None`.

            If `wait_for_completion` is `False`, returns a
            :class:`~zhmcclient.Job` object representing the asynchronously
            executing job on the HMC.

        Raises:

          :exc:`~zhmcclient.HTTPError`
          :exc:`~zhmcclient.ParseError`
          :exc:`~zhmcclient.AuthError`
          :exc:`~zhmcclient.ConnectionError`
          :exc:`~zhmcclient.OperationTimeout`: The timeout expired while
            waiting for completion of the operation.
          :exc:`~zhmcclient.StatusTimeout`: The timeout expired while
            waiting for the desired LPAR status.
        """
        body = {}
        result = self.manager.session.post(
            self.uri + '/operations/activate',
            body,
            wait_for_completion=wait_for_completion,
            operation_timeout=operation_timeout)
        if wait_for_completion:
            self._wait_for_status("not-operating", status_timeout)
        return result

    @logged_api_call
    def deactivate(self, wait_for_completion=True,
                   operation_timeout=None, status_timeout=None):
        """
        De-activate (stop) this LPAR, using the HMC operation "Deactivate
        Logical Partition".

        This HMC operation has deferred status behavior: If the asynchronous
        job on the HMC is complete, it takes a few seconds until the LPAR
        status has reached the desired value. If `wait_for_completion=True`,
        this method repeatedly checks the status of the LPAR after the HMC
        operation has completed, and waits until the status is in the desired
        state "not-activated".

        Authorization requirements:

        * Object-access permission to the CPC containing this LPAR.
        * Object-access permission to this LPAR.
        * Task permission for the "Deactivate" task.

        Parameters:

          wait_for_completion (bool):
            Boolean controlling whether this method should wait for completion
            of the requested asynchronous HMC operation, as follows:

            * If `True`, this method will wait for completion of the
              asynchronous job performing the operation, and for the status
              becoming "non-activated".

            * If `False`, this method will return immediately once the HMC has
              accepted the request to perform the operation.

          operation_timeout (:term:`number`):
            Timeout in seconds, for waiting for completion of the asynchronous
            job performing the operation. The special value 0 means that no
            timeout is set. `None` means that the default async operation
            timeout of the session is used. If the timeout expires when
            `wait_for_completion=True`, a
            :exc:`~zhmcclient.OperationTimeout` is raised.

          status_timeout (:term:`number`):
            Timeout in seconds, for waiting that the status of the LPAR has
            reached the desired status, after the HMC operation has completed.
            The special value 0 means that no timeout is set. `None` means that
            the default async operation timeout of the session is used.
            If the timeout expires when `wait_for_completion=True`, a
            :exc:`~zhmcclient.StatusTimeout` is raised.

        Returns:

          `None` or :class:`~zhmcclient.Job`:

            If `wait_for_completion` is `True`, returns `None`.

            If `wait_for_completion` is `False`, returns a
            :class:`~zhmcclient.Job` object representing the asynchronously
            executing job on the HMC.

        Raises:

          :exc:`~zhmcclient.HTTPError`
          :exc:`~zhmcclient.ParseError`
          :exc:`~zhmcclient.AuthError`
          :exc:`~zhmcclient.ConnectionError`
          :exc:`~zhmcclient.OperationTimeout`: The timeout expired while
            waiting for completion of the operation.
          :exc:`~zhmcclient.StatusTimeout`: The timeout expired while
            waiting for the desired LPAR status.
        """
        body = {'force': True}
        result = self.manager.session.post(
            self.uri + '/operations/deactivate',
            body,
            wait_for_completion=wait_for_completion,
            operation_timeout=operation_timeout)
        if wait_for_completion:
            self._wait_for_status("not-activated", status_timeout)
        return result

    @logged_api_call
    def load(self, load_address, load_parameter="", wait_for_completion=True,
             operation_timeout=None, status_timeout=None):
        """
        Load (boot) this LPAR from a load address (boot device), using the HMC
        operation "Load Logical Partition".

        This HMC operation has deferred status behavior: If the asynchronous
        job on the HMC is complete, it takes a few seconds until the LPAR
        status has reached the desired value. If `wait_for_completion=True`,
        this method repeatedly checks the status of the LPAR after the HMC
        operation has completed, and waits until the status is in the desired
        state "operating".

        Authorization requirements:

        * Object-access permission to the CPC containing this LPAR.
        * Object-access permission to this LPAR.
        * Task permission for the "Load" task.

        Parameters:

          load_address (:term:`string`): Device number of the boot device.

          load_parameter (:term:`string`): Optional load control string.

          wait_for_completion (bool):
            Boolean controlling whether this method should wait for completion
            of the requested asynchronous HMC operation, as follows:

            * If `True`, this method will wait for completion of the
              asynchronous job performing the operation, and for the status
              becoming "operating".

            * If `False`, this method will return immediately once the HMC has
              accepted the request to perform the operation.

          operation_timeout (:term:`number`):
            Timeout in seconds, for waiting for completion of the asynchronous
            job performing the operation. The special value 0 means that no
            timeout is set. `None` means that the default async operation
            timeout of the session is used. If the timeout expires when
            `wait_for_completion=True`, a
            :exc:`~zhmcclient.OperationTimeout` is raised.

          status_timeout (:term:`number`):
            Timeout in seconds, for waiting that the status of the LPAR has
            reached the desired status, after the HMC operation has completed.
            The special value 0 means that no timeout is set. `None` means that
            the default async operation timeout of the session is used.
            If the timeout expires when `wait_for_completion=True`, a
            :exc:`~zhmcclient.StatusTimeout` is raised.

        Returns:

          `None` or :class:`~zhmcclient.Job`:

            If `wait_for_completion` is `True`, returns `None`.

            If `wait_for_completion` is `False`, returns a
            :class:`~zhmcclient.Job` object representing the asynchronously
            executing job on the HMC.

        Raises:

          :exc:`~zhmcclient.HTTPError`
          :exc:`~zhmcclient.ParseError`
          :exc:`~zhmcclient.AuthError`
          :exc:`~zhmcclient.ConnectionError`
          :exc:`~zhmcclient.OperationTimeout`: The timeout expired while
            waiting for completion of the operation.
          :exc:`~zhmcclient.StatusTimeout`: The timeout expired while
            waiting for the desired LPAR status.
        """
        body = {'load-address': load_address}
        if load_parameter != "":
            body['load-parameter'] = load_parameter
        result = self.manager.session.post(
            self.uri + '/operations/load',
            body,
            wait_for_completion=wait_for_completion,
            operation_timeout=operation_timeout)
        if wait_for_completion:
            self._wait_for_status("operating", status_timeout)
        return result

    @logged_api_call
    def open_os_message_channel(self, include_refresh_messages=True):
        """
        Open a JMS message channel to this LPAR's operating system, returning
        the string "topic" representing the message channel.

        Parameters:

          include_refresh_messages (bool):
            Boolean controlling whether refresh operating systems messages
            should be sent, as follows:

            * If `True`, refresh messages will be recieved when the user
              connects to the topic. The default.

            * If `False`, refresh messages will not be recieved when the user
              connects to the topic.

        Returns:

          :term:`string`:

            Returns a string representing the os-message-notification JMS
            topic. The user can connect to this topic to start the flow of
            operating system messages.

        Raises:

          :exc:`~zhmcclient.HTTPError`
          :exc:`~zhmcclient.ParseError`
          :exc:`~zhmcclient.AuthError`
          :exc:`~zhmcclient.ConnectionError`
        """
        body = {'include-refresh-messages': include_refresh_messages}
        result = self.manager.session.post(
            self.uri + '/operations/open-os-message-channel', body)
        return result['topic-name']

    @logged_api_call
    def send_os_command(self, os_command_text, is_priority=False):
        """
        Send a command to the operating system running in this LPAR.

        Parameters:

          os_command_text (string): The text of the operating system command.

          is_priority (bool):
            Boolean controlling whether this is a priority operating system
            command, as follows:

            * If `True`, this message is treated as a priority operating
              system command.

            * If `False`, this message is not treated as a priority
              operating system command. The default.

        Returns:

          None

        Raises:

          :exc:`~zhmcclient.HTTPError`
          :exc:`~zhmcclient.ParseError`
          :exc:`~zhmcclient.AuthError`
          :exc:`~zhmcclient.ConnectionError`
        """
        body = {'is-priority': is_priority,
                'operating-system-command-text': os_command_text}
        self.manager.session.post(
            self.uri + '/operations/send-os-cmd', body)

    def _wait_for_status(self, status, status_timeout=None):
        """
        Wait until the status of this LPAR has a desired value.

        Parameters:

          status (:term:`string`):
            Desired LPAR status(es) to reach; one of the following values:
            * ``"not-activated"`` - The LPAR is not active.
            * ``"not-operating" - The LPAR is active but no operating system is
              running in the LPAR.
            * ``"operating"`` - The LPAR is active and an operating system is
              running in the LPAR.

            The fourth possible LPAR status value ``"exceptions"`` indicates
            that the LPAR or its CPC has one or more unusual conditions. This
            value should not normally be specified for this parameter. Each
            time the ``"exceptions"`` status is seen by this method, a log
            entry at the info level is written to record that, but otherwise
            it is treated like other undesired suatus values.

            Note that the description of LPAR status values in the
            :term:`HMC API` book (as of its version 2.13.1) is partly
            confusing.

          status_timeout (:term:`number`):
            Timeout in seconds, for waiting that the status of the LPAR has
            reached the desired status. The special value 0 means that no
            timeout is set.
            If the timeout expires when `wait_for_completion=True`, a
            :exc:`~zhmcclient.StatusTimeout` is raised.

        Raises:

          :exc:`~zhmcclient.HTTPError`
          :exc:`~zhmcclient.ParseError`
          :exc:`~zhmcclient.AuthError`
          :exc:`~zhmcclient.ConnectionError`
          :exc:`~zhmcclient.StatusTimeout`: The timeout expired while
            waiting for the desired LPAR status.
        """
        if status_timeout is None:
            status_timeout = \
                self.manager.session.retry_timeout_config.status_timeout
        if status_timeout > 0:
            end_time = time.time() + status_timeout
        while True:

            # Fastest way to get actual status value:
            lpars = self.manager.cpc.lpars.list(
                filter_args={'name': self.name})
            assert len(lpars) == 1
            this_lpar = lpars[0]
            actual_status = this_lpar.get_property('status')

            if actual_status == status:
                return
            elif actual_status == "exceptions":
                LOG.info("LPAR {} in CPC {} has status 'exceptions' while "
                         "waiting for status '{}' (if no other errors follow, "
                         "this was a temporary condition)".
                         format(this_lpar.name, this_lpar.manager.cpc.name,
                                status))

            if status_timeout > 0 and time.time() > end_time:
                raise StatusTimeout(
                    "Waiting for LPAR {} to reach status '{}' timed out after "
                    "{} s - current status is '{}'".
                    format(self.name, status, status_timeout, actual_status))

            time.sleep(1)  # Avoid hot spin loop



#!/usr/bin/env python
# Copyright 2016 IBM Corp. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Unit tests for _adapter module.
"""

from __future__ import absolute_import, print_function

import unittest
import requests_mock

from zhmcclient import Session, Client, Adapter


class AdapterTests(unittest.TestCase):
    """All tests for Adapter and AdapterManager classes."""

    def setUp(self):
        self.session = Session('adapter-dpm-host', 'adapter-user',
                               'adapter-pwd')
        self.client = Client(self.session)
        with requests_mock.mock() as m:
            # Because logon is deferred until needed, we perform it
            # explicitly in order to keep mocking in the actual test simple.
            m.post('/api/sessions', json={'api-session': 'adapter-session-id'})
            self.session.logon()

        self.cpc_mgr = self.client.cpcs
        with requests_mock.mock() as m:
            result = {
                'cpcs': [
                    {
                        'object-uri': '/api/cpcs/adapter-cpc-id-1',
                        'name': 'CPC',
                        'status': 'service-required',
                    }
                ]
            }
            m.get('/api/cpcs', json=result)

            cpcs = self.cpc_mgr.list()
            self.cpc = cpcs[0]

    def tearDown(self):
        with requests_mock.mock() as m:
            m.delete('/api/sessions/this-session', status_code=204)
            self.session.logoff()

    def test_init(self):
        """Test __init__() on AdapterManager instance in CPC."""
        adapter_mgr = self.cpc.adapters
        self.assertEqual(adapter_mgr.cpc, self.cpc)

    def test_list_short_ok(self):
        """
        Test successful list() with short set of properties on AdapterManager
        instance in CPC.
        """
        adapter_mgr = self.cpc.adapters
        with requests_mock.mock() as m:
            result = {
                'adapters': [
                    {
                        'adapter-family': 'ficon',
                        'adapter-id': '18C',
                        'type': 'fcp',
                        'status': 'active',
                        'object-uri': '/api/adapters/fake-adapter-id-1',
                        'name': 'FCP Adapter 1'
                    },
                    {
                        'adapter-family': 'osa',
                        'adapter-id': '1C4',
                        'type': 'osd',
                        'status': 'active',
                        'object-uri': '/api/adapters/fake-adapter-id-2',
                        'name': 'OSD Adapter 1'
                    }
                ]
            }

            m.get('/api/cpcs/adapter-cpc-id-1/adapters', json=result)

            adapters = adapter_mgr.list(full_properties=False)

            self.assertEqual(len(adapters), len(result['adapters']))
            for idx, adapter in enumerate(adapters):
                self.assertEqual(
                    adapter.properties,
                    result['adapters'][idx])
                self.assertEqual(
                    adapter.uri,
                    result['adapters'][idx]['object-uri'])
                self.assertFalse(adapter.full_properties)
                self.assertEqual(adapter.manager, adapter_mgr)

    def test_list_full_ok(self):
        """
        Test successful list() with full set of properties on AdapterManager
        instance in CPC.
        """
        adapter_mgr = self.cpc.adapters
        with requests_mock.mock() as m:
            result = {
                'adapters': [
                    {
                        'adapter-family': 'ficon',
                        'adapter-id': '18C',
                        'type': 'fcp',
                        'status': 'active',
                        'object-uri': '/api/adapters/fake-adapter-id-1',
                        'name': 'FCP Adapter 1'
                    },
                    {
                        'adapter-family': 'osa',
                        'adapter-id': '1C4',
                        'type': 'osd',
                        'status': 'active',
                        'object-uri': '/api/adapters/fake-adapter-id-2',
                        'name': 'OSD Adapter 1'
                    }
                ]
            }

            m.get('/api/cpcs/adapter-cpc-id-1/adapters', json=result)

            mock_result_adapter1 = {
                'adapter-family': 'ficon',
                'adapter-id': '18C',
                'type': 'fcp',
                'status': 'active',
                'object-uri': '/api/adapters/fake-adapter-id-1',
                'name': 'FCP Adapter 1',
                'more_properties': 'bliblablub'
            }
            m.get('/api/adapters/fake-adapter-id-1',
                  json=mock_result_adapter1)
            mock_result_adapter2 = {
                'adapter-family': 'osa',
                'adapter-id': '1C4',
                'type': 'osd',
                'status': 'active',
                'object-uri': '/api/adapters/fake-adapter-id-2',
                'name': 'OSD Adapter 1',
                'description': 'Test Adapter',
                'more_properties': 'bliblablub'
            }
            m.get('/api/adapters/fake-adapter-id-2',
                  json=mock_result_adapter2)

            adapters = adapter_mgr.list(full_properties=True)

            self.assertEqual(len(adapters), len(result['adapters']))
            for idx, adapter in enumerate(adapters):
                self.assertEqual(adapter.properties['name'],
                                 result['adapters'][idx]['name'])
                self.assertEqual(
                    adapter.uri,
                    result['adapters'][idx]['object-uri'])
                self.assertTrue(adapter.full_properties)
                self.assertEqual(adapter.manager, adapter_mgr)

    def test_create_hipersocket(self):
        """
        This tests the 'Create Hipersocket' operation.
        """
        adapter_mgr = self.cpc.adapters
        with requests_mock.mock() as m:
            result = {
                'object-uri': '/api/adapters/fake-adapter-id-1'
            }
            m.post('/api/cpcs/adapter-cpc-id-1/adapters', json=result)

            adapter = adapter_mgr.create_hipersocket(properties={})

            self.assertTrue(isinstance(adapter, Adapter))
            self.assertEqual(adapter.properties, result)
            self.assertEqual(adapter.uri, result['object-uri'])

    def test_delete(self):
        """
        This tests the 'Delete Adapter' operation.
        """
        adapter_mgr = self.cpc.adapters
        with requests_mock.mock() as m:
            result = {
                'adapters': [
                    {
                        'adapter-family': 'ficon',
                        'adapter-id': '18C',
                        'type': 'fcp',
                        'status': 'active',
                        'object-uri': '/api/adapters/fake-adapter-id-1',
                        'name': 'FCP Adapter 1'
                    },
                    {
                        'adapter-family': 'osa',
                        'adapter-id': '1C4',
                        'type': 'osd',
                        'status': 'active',
                        'object-uri': '/api/adapters/fake-adapter-id-2',
                        'name': 'OSD Adapter 1'
                    }
                ]
            }
            m.get('/api/cpcs/adapter-cpc-id-1/adapters', json=result)

            adapters = adapter_mgr.list(full_properties=False)
            adapter = adapters[0]
            m.delete(
                "/api/adapters/fake-adapter-id-1",
                json=result)
            status = adapter.delete()
            self.assertEqual(status, None)

    def test_update_properties(self):
        """
        This tests the 'Update Adapter Properties' operation.
        """
        adapter_mgr = self.cpc.adapters
        with requests_mock.mock() as m:
            result = {
                'adapters': [
                    {
                        'adapter-family': 'ficon',
                        'adapter-id': '18C',
                        'type': 'fcp',
                        'status': 'active',
                        'object-uri': '/api/adapters/fake-adapter-id-1',
                        'name': 'FCP Adapter 1'
                    },
                    {
                        'adapter-family': 'osa',
                        'adapter-id': '1C4',
                        'type': 'osd',
                        'status': 'active',
                        'object-uri': '/api/adapters/fake-adapter-id-2',
                        'name': 'OSD Adapter 1'
                    }
                ]
            }
            m.get('/api/cpcs/adapter-cpc-id-1/adapters', json=result)

            adapters = adapter_mgr.list(full_properties=False)
            adapter = adapters[0]
            m.post(
                "/api/adapters/fake-adapter-id-1",
                json=result)
            status = adapter.update_properties(properties={})
            self.assertEqual(status, None)


if __name__ == '__main__':
    unittest.main()



# Copyright 2016 IBM Corp. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Unit tests for _logging module.
"""

from __future__ import absolute_import, print_function

import logging
import unittest
from testfixtures import log_capture

from zhmcclient._logging import logged_api_call


#
# Various uses of the @logged_api_call decorator
#

@logged_api_call
def decorated_global_function():
    """A decorated function at the global (module) level."""
    pass


def global1_function():

    @logged_api_call
    def decorated_inner1_function():
        """A decorated inner function defined in a global function."""
        pass

    return decorated_inner1_function


def get_decorated_inner1_function():
    return global1_function()


def global2_function():

    def inner1_function():

        @logged_api_call
        def decorated_inner2_function():
            """A decorated inner function defined in another inner function
            that is defined in a global function."""
            pass

        return decorated_inner2_function

    return inner1_function()


def get_decorated_inner2_function():
    return global2_function()


class Decorator1Class(object):

    @logged_api_call
    def decorated_method(self):
        """A decorated method of a class."""
        pass


class Decorator2Class(object):

    @staticmethod
    def method():

        @logged_api_call
        def decorated_inner_function():
            """A decorated inner function defined in a method of a class."""
            pass

        return decorated_inner_function

    @staticmethod
    def get_decorated_inner_function():
        return Decorator2Class.method()


#
# Supporting definitions
#

class CallerClass(object):

    @staticmethod
    def call_from_method(func, *args, **kwargs):
        """
        A supporting method that calls the specified function with the
        specified arguments and keyword arguments. This is used by the test
        cases so that this function acts as a caller for the decorated API
        function.
        """
        return func(*args, **kwargs)


def call_from_global(func, *args, **kwargs):
    """
    A supporting global function that calls the specified function with the
    specified arguments and keyword arguments. This is used by the test cases
    so that this function acts as a caller for the decorated API function.
    """
    return func(*args, **kwargs)


# Some expected values that are constant
_EXP_LOGGER_NAME = 'zhmcclient.api'
_EXP_LOG_LEVEL = 'DEBUG'
_EXP_LOG_MSG_ENTER = "==> %s, args: %.500r, kwargs: %.500r"
_EXP_LOG_MSG_LEAVE = "<== %s, result: %.1000r"


#
# Test cases
#

class TestLogging(unittest.TestCase):
    """All test cases for the @logged_api_call decorator."""

    def assert_log_capture(self, log_capture, exp_apifunc):

        self.assertEqual(len(log_capture.records), 2)

        enter_record = log_capture.records[0]
        self.assertEqual(enter_record.name, _EXP_LOGGER_NAME)
        self.assertEqual(enter_record.levelname, _EXP_LOG_LEVEL)
        self.assertEqual(enter_record.msg, _EXP_LOG_MSG_ENTER)
        self.assertEqual(enter_record.args[0], exp_apifunc)
        # We don't check the positional args and keyword args

        leave_record = log_capture.records[1]
        self.assertEqual(leave_record.name, _EXP_LOGGER_NAME)
        self.assertEqual(leave_record.levelname, _EXP_LOG_LEVEL)
        self.assertEqual(leave_record.msg, _EXP_LOG_MSG_LEAVE)
        self.assertEqual(leave_record.args[0], exp_apifunc)
        # We don't check the positional args and keyword args

    @log_capture(level=logging.DEBUG)
    def test_1a_global_from_global(self, capture):
        """Simple test calling a decorated global function from a global
        function."""

        call_from_global(decorated_global_function)

        self.assert_log_capture(capture, 'decorated_global_function()')

    @log_capture(level=logging.DEBUG)
    def test_1b_global_from_method(self, capture):
        """Simple test calling a decorated global function from a method."""

        CallerClass().call_from_method(decorated_global_function)

        self.assert_log_capture(capture, 'decorated_global_function()')

    @log_capture(level=logging.DEBUG)
    def test_2a_global_inner1_from_global(self, capture):
        """Simple test calling a decorated inner function defined in a global
        function from a global function."""

        decorated_inner1_function = get_decorated_inner1_function()

        call_from_global(decorated_inner1_function)

        self.assert_log_capture(capture,
                                'global1_function.decorated_inner1_function()')

    @log_capture(level=logging.DEBUG)
    def test_2b_global_inner1_from_method(self, capture):
        """Simple test calling a decorated inner function defined in a global
        function from a method."""

        decorated_inner1_function = get_decorated_inner1_function()

        CallerClass().call_from_method(decorated_inner1_function)

        self.assert_log_capture(capture,
                                'global1_function.decorated_inner1_function()')

    @log_capture(level=logging.DEBUG)
    def test_3a_global_inner2_from_global(self, capture):
        """Simple test calling a decorated inner function defined in an inner
        function defined in a global function from a global function."""

        decorated_inner2_function = get_decorated_inner2_function()

        call_from_global(decorated_inner2_function)

        self.assert_log_capture(capture,
                                'inner1_function.decorated_inner2_function()')

    @log_capture(level=logging.DEBUG)
    def test_3b_global_inner1_from_method(self, capture):
        """Simple test calling a decorated inner function defined in an inner
        function defined in a global function from a method."""

        decorated_inner2_function = get_decorated_inner2_function()

        CallerClass().call_from_method(decorated_inner2_function)

        self.assert_log_capture(capture,
                                'inner1_function.decorated_inner2_function()')

    @log_capture(level=logging.DEBUG)
    def test_4a_method_from_global(self, capture):
        """Simple test calling a decorated method from a global function."""

        decorated_method = Decorator1Class.decorated_method
        d = Decorator1Class()

        call_from_global(decorated_method, d)

        self.assert_log_capture(capture, 'Decorator1Class.decorated_method()')

    @log_capture(level=logging.DEBUG)
    def test_4b_method_from_method(self, capture):
        """Simple test calling a decorated method from a method."""

        decorated_method = Decorator1Class.decorated_method
        d = Decorator1Class()

        CallerClass().call_from_method(decorated_method, d)

        self.assert_log_capture(capture, 'Decorator1Class.decorated_method()')

    @log_capture(level=logging.DEBUG)
    def test_5a_method_from_global(self, capture):
        """Simple test calling a decorated inner function defined in a method
        from a global function."""

        decorated_inner_function = \
            Decorator2Class.get_decorated_inner_function()

        call_from_global(decorated_inner_function)

        self.assert_log_capture(capture,
                                'method.decorated_inner_function()')

    @log_capture(level=logging.DEBUG)
    def test_5b_method_from_method(self, capture):
        """Simple test calling a decorated inner function defined in a method
        from a method."""

        decorated_inner_function = \
            Decorator2Class.get_decorated_inner_function()

        CallerClass().call_from_method(decorated_inner_function)

        self.assert_log_capture(capture,
                                'method.decorated_inner_function()')

# TODO: Add test cases for _get_logger(), specifically for null-handler



#!/usr/bin/env python
# Copyright 2016 IBM Corp. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Example shows how to find an LPAR in a CPC
and activate/deactivate/load of an LPAR.
"""

import sys
import logging
import yaml
import time
import requests.packages.urllib3

import zhmcclient

requests.packages.urllib3.disable_warnings()

if len(sys.argv) != 2:
    print("Usage: %s hmccreds.yaml" % sys.argv[0])
    sys.exit(2)
hmccreds_file = sys.argv[1]

with open(hmccreds_file, 'r') as fp:
    hmccreds = yaml.load(fp)

examples = hmccreds.get("examples", None)
if examples is None:
    print("examples not found in credentials file %s" % \
          (hmccreds_file))
    sys.exit(1)

lpar_operations = examples.get("lpar_operations", None)
if lpar_operations is None:
    print("lpar_operations not found in credentials file %s" % \
          (hmccreds_file))
    sys.exit(1)

loglevel = lpar_operations.get("loglevel", None)
if loglevel is not None:
    level = getattr(logging, loglevel.upper(), None)
    if level is None:
        print("Invalid value for loglevel in credentials file %s: %s" % \
              (hmccreds_file, loglevel))
        sys.exit(1)
    logging.basicConfig(level=level)

hmc = lpar_operations["hmc"]
cpcname = lpar_operations["cpcname"]
lparname = lpar_operations["lparname"]
loaddev = lpar_operations["loaddev"]
deactivate = lpar_operations["deactivate"]

cred = hmccreds.get(hmc, None)
if cred is None:
    print("Credentials for HMC %s not found in credentials file %s" % \
          (hmc, hmccreds_file))
    sys.exit(1)

userid = cred['userid']
password = cred['password']

print(__doc__)

print("Using HMC %s with userid %s ..." % (hmc, userid))
session = zhmcclient.Session(hmc, userid, password)
cl = zhmcclient.Client(session)

timestats = lpar_operations.get("timestats", None)
if timestats:
    session.time_stats_keeper.enable()

retries = 10

print("Finding CPC by name=%s ..." % cpcname)
try:
    cpc = cl.cpcs.find(name=cpcname)
except zhmcclient.NotFound:
    print("Could not find CPC %s on HMC %s" % (cpcname, hmc))
    sys.exit(1)
print("Found CPC %s at: %s" % (cpc.name, cpc.uri))

print("Finding LPAR by name=%s ..." % lparname)
# We use list() instead of find() because find(name=..) is optimized by using
# the name-to-uri cache and therefore returns an Lpar object with only a
# minimal set of properties, and particularly no 'status' property.
# That would drive an extra "Get Logical Partition Properties" operation when
# the status property is accessed.
lpars = cpc.lpars.list(filter_args={'name': lparname})
if len(lpars) != 1:
    print("Could not find LPAR %s in CPC %s" % (lparname, cpc.name))
    sys.exit(1)
lpar = lpars[0]
print("Found LPAR %s at: %s" % (lpar.name, lpar.uri))

status = lpar.get_property('status')
print("Status of LPAR %s: %s" % (lpar.name, status))

if status != "not-activated":
    print("Deactivating LPAR %s ..." % lpar.name)
    lpar.deactivate()
    for i in range(0, retries):
        print("Refreshing ...")
        lpar = cpc.lpars.list(filter_args={'name': lparname})[0]
        status = lpar.get_property('status')
        print("Status of LPAR %s: %s" % (lpar.name, status))
        if status == 'not-activated':
            break
        time.sleep(1)
    else:
        print("Warning: After %d retries, status of LPAR %s after Deactivate "
              "is still: %s" % (retries, lpar.name, status))

print("Activating LPAR %s ..." % lpar.name)
lpar.activate()
for i in range(0, retries):
    print("Refreshing ...")
    lpar = cpc.lpars.list(filter_args={'name': lparname})[0]
    status = lpar.get_property('status')
    print("Status of LPAR %s: %s" % (lpar.name, status))
    if status == 'not-operating':
        break
    time.sleep(1)
else:
    print("Warning: After %d retries, status of LPAR %s after Activate "
          "is still: %s" % (retries, lpar.name, status))

print("Loading LPAR %s from device %s ..." % (lpar.name, loaddev))
lpar.load(loaddev)
for i in range(0, retries):
    print("Refreshing ...")
    lpar = cpc.lpars.list(filter_args={'name': lparname})[0]
    status = lpar.get_property('status')
    print("Status of LPAR %s: %s" % (lpar.name, status))
    if status == 'operating':
        break
    time.sleep(1)
else:
    print("Warning: After %d retries, status of LPAR %s after Load "
          "is still: %s" % (retries, lpar.name, status))

if deactivate == "yes":
    print("Deactivating LPAR %s ..." % lpar.name)
    lpar.deactivate()
    for i in range(0, retries):
        print("Refreshing ...")
        lpar = cpc.lpars.list(filter_args={'name': lparname})[0]
        status = lpar.get_property('status')
        print("Status of LPAR %s: %s" % (lpar.name, status))
        if status == 'not-activated':
            break
        time.sleep(1)
    else:
        print("Warning: After %d retries, status of LPAR %s after Deactivate "
              "is still: %s" % (retries, lpar.name, status))

print("Logging off ...")
session.logoff()

if timestats:
    print(session.time_stats_keeper)

print("Done.")



# Copyright 2016 IBM Corp. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
zhmcclient - A pure Python client library for the z Systems HMC Web Services
API.

For documentation, see TODO: Add link to RTD once available.
"""

from __future__ import absolute_import

from ._version import *       # noqa: F401
from ._constants import *     # noqa: F401
from ._exceptions import *    # noqa: F401
from ._manager import *       # noqa: F401
from ._resource import *      # noqa: F401
from ._logging import *       # noqa: F401
from ._session import *       # noqa: F401
from ._timestats import *     # noqa: F401
from ._client import *        # noqa: F401
from ._cpc import *           # noqa: F401
from ._lpar import *          # noqa: F401
from ._partition import *     # noqa: F401
from ._activation_profile import *     # noqa: F401
from ._adapter import *       # noqa: F401
from ._nic import *           # noqa: F401
from ._hba import *           # noqa: F401
from ._virtual_function import *       # noqa: F401
from ._virtual_switch import *         # noqa: F401
from ._port import *          # noqa: F401
from ._notification import *  # noqa: F401



# Copyright 2016 IBM Corp. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import absolute_import

import click

import zhmcclient
from .zhmccli import cli


@cli.group('session')
def session_group():
    """
    Command group for managing sessions.

    In addition to the command-specific options shown in this help text, the
    general options (see 'zhmc --help') can also be specified before the
    command.
    """


@session_group.command('create')
@click.pass_obj
def session_create(cmd_ctx):
    """
    Create an HMC session.

    In addition to the command-specific options shown in this help text, the
    general options (see 'zhmc --help') can also be specified before the
    command.
    """
    cmd_ctx.execute_cmd(lambda: cmd_session_create(cmd_ctx))


@session_group.command('delete')
@click.pass_obj
def session_delete(cmd_ctx):
    """
    Delete the current HMC session.

    In addition to the command-specific options shown in this help text, the
    general options (see 'zhmc --help') can also be specified before the
    command.
    """
    cmd_ctx.execute_cmd(lambda: cmd_session_delete(cmd_ctx))


def cmd_session_create(cmd_ctx):
    """Create an HMC session."""
    session = cmd_ctx.session
    try:
        session.logon(verify=True)
    except zhmcclient.Error as exc:
        raise click.ClickException("%s: %s" % (exc.__class__.__name__, exc))

    cmd_ctx.spinner.stop()
    click.echo("export ZHMC_HOST=%s" % session.host)
    click.echo("export ZHMC_USERID=%s" % session.userid)
    click.echo("export ZHMC_SESSION_ID=%s" % session.session_id)


def cmd_session_delete(cmd_ctx):
    """Delete the current HMC session."""
    session = cmd_ctx.session
    try:
        session.logoff()
    except zhmcclient.Error as exc:
        raise click.ClickException("%s: %s" % (exc.__class__.__name__, exc))

    cmd_ctx.spinner.stop()
    click.echo("unset ZHMC_SESSION_ID")



#!/usr/bin/env python
# Copyright 2017 IBM Corp. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Unit test cases for the _idpool module of the zhmcclient_mock package.
"""

from __future__ import absolute_import, print_function

import requests.packages.urllib3
import unittest

from zhmcclient_mock._idpool import IdPool


class IdPoolTests(unittest.TestCase):
    """All tests for class IdPool."""

    def test_init_error_1(self):

        with self.assertRaises(ValueError):
            IdPool(7, 6)

    def test_invalid_free_error_1(self):

        pool = IdPool(5, 5)

        with self.assertRaises(ValueError):
            pool.free(4)  # not in range

        with self.assertRaises(ValueError):
            pool.free(5)  # in range but not allocated

        with self.assertRaises(ValueError):
            pool.free(6)  # not in range

    def test_invalid_free_error_2(self):

        pool = IdPool(5, 5)

        pool.free_if_allocated(4)  # not in range (= not allocated)

        pool.free_if_allocated(5)  # in range but not allocated

        pool.free_if_allocated(6)  # not in range (= not allocated)

    def _test_exhausting_for_lo_hi(self, lowest, highest):
        start = lowest
        end = highest + 1

        pool = IdPool(lowest, highest)

        # Exhaust the pool
        id_list = []
        for i in range(start, end):
            id = pool.alloc()
            id_list.append(id)

        # Verify uniqueness of the ID values
        id_set = set(id_list)
        self.assertEqual(len(id_set), len(id_list))

        # Verify that the pool is exhausted
        with self.assertRaises(ValueError):
            pool.alloc()

    def _test_free_for_lo_hi(self, lowest, highest):
        start = lowest
        end = highest + 1

        pool = IdPool(lowest, highest)

        # Exhaust the pool
        id_list1 = []
        for i in range(start, end):
            id = pool.alloc()
            id_list1.append(id)

        # Return everything to the pool
        for id in id_list1:
            pool.free(id)

        # Verify that nothing is used in the pool
        self.assertEqual(len(pool._used), 0)

        # Exhaust the pool
        id_list2 = []
        for i in range(start, end):
            id = pool.alloc()
            id_list2.append(id)

        # Verify that the same ID values came back as last time
        self.assertEqual(set(id_list1), set(id_list2))

        # Verify that the pool is exhausted
        with self.assertRaises(ValueError):
            pool.alloc()

    def _test_all_for_lo_hi(self, lowest, highest):
        self._test_exhausting_for_lo_hi(lowest, highest)
        self._test_free_for_lo_hi(lowest, highest)

    def test_all(self):
        # Knowing that the chunk size is 10, we focus on the sizes and range
        # boundaries around that
        self._test_all_for_lo_hi(0, 0)
        self._test_all_for_lo_hi(0, 1)
        self._test_all_for_lo_hi(0, 9)
        self._test_all_for_lo_hi(0, 10)
        self._test_all_for_lo_hi(0, 11)
        self._test_all_for_lo_hi(0, 19)
        self._test_all_for_lo_hi(0, 20)
        self._test_all_for_lo_hi(0, 21)
        self._test_all_for_lo_hi(3, 3)
        self._test_all_for_lo_hi(3, 4)
        self._test_all_for_lo_hi(3, 9)
        self._test_all_for_lo_hi(3, 10)
        self._test_all_for_lo_hi(3, 11)
        self._test_all_for_lo_hi(3, 12)
        self._test_all_for_lo_hi(3, 13)
        self._test_all_for_lo_hi(3, 14)
        self._test_all_for_lo_hi(9, 9)
        self._test_all_for_lo_hi(9, 10)
        self._test_all_for_lo_hi(9, 11)
        self._test_all_for_lo_hi(9, 18)
        self._test_all_for_lo_hi(9, 19)
        self._test_all_for_lo_hi(9, 20)
        self._test_all_for_lo_hi(10, 10)
        self._test_all_for_lo_hi(10, 11)
        self._test_all_for_lo_hi(10, 19)
        self._test_all_for_lo_hi(10, 20)
        self._test_all_for_lo_hi(10, 21)
        self._test_all_for_lo_hi(11, 11)
        self._test_all_for_lo_hi(11, 12)
        self._test_all_for_lo_hi(11, 20)
        self._test_all_for_lo_hi(11, 21)
        self._test_all_for_lo_hi(11, 22)


if __name__ == '__main__':
    requests.packages.urllib3.disable_warnings()
    unittest.main()



#!/usr/bin/env python
# Copyright 2016 IBM Corp. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Tests for time statistics (`_timestats` module).
"""

from __future__ import absolute_import, print_function

import time
import unittest

from zhmcclient import TimeStatsKeeper


PRINT_HEADER = \
    "Time statistics (times in seconds):\n" \
    "Count  Average  Minimum  Maximum  Operation name"

PRINT_HEADER_DISABLED = \
    "Time statistics (times in seconds):\n" \
    "Disabled."


def time_abs_delta(t1, t2):
    """
    Return the positive difference between two float values.
    """
    return abs(t1 - t2)


def measure(stats, duration):
    """
    Return the measured duration of one begin/end cycle of a TimeStats object
    of a given intended duration.  The runtime behavior may be that the actual
    duration is larger than the intended duration, therefore we need to measure
    the actual duration.
    """
    begin = time.time()
    stats.begin()
    time.sleep(duration)
    stats.end()
    end = time.time()
    return end - begin


class TimeStatsTests(unittest.TestCase):
    """All tests for TimeStatsKeeper and TimeStats."""

    def test_enabling(self):
        """Test enabling and disabling."""

        keeper = TimeStatsKeeper()

        self.assertFalse(keeper.enabled,
                         "Verify that initial state is disabled")

        keeper.disable()
        self.assertFalse(keeper.enabled,
                         "Verify that disabling a disabled keeper works")

        keeper.enable()
        self.assertTrue(keeper.enabled,
                        "Verify that enabling a disabled keeper works")

        keeper.enable()
        self.assertTrue(keeper.enabled,
                        "Verify that enabling an enabled keeper works")

        keeper.disable()
        self.assertFalse(keeper.enabled,
                         "Verify that disabling an enabled keeper works")

    def test_get(self):
        """Test getting time statistics."""

        keeper = TimeStatsKeeper()
        snapshot_length = len(keeper.snapshot())
        self.assertEqual(snapshot_length, 0,
                         "Verify that initial state has no time statistics. "
                         "Actual number = %d" % snapshot_length)

        stats = keeper.get_stats('foo')
        snapshot_length = len(keeper.snapshot())
        self.assertEqual(snapshot_length, 0,
                         "Verify that getting a new stats with a disabled "
                         "keeper results in no time statistics. "
                         "Actual number = %d" % snapshot_length)
        self.assertEqual(stats.keeper, keeper)
        self.assertEqual(stats.name, "disabled")  # stats for disabled keeper
        self.assertEqual(stats.count, 0)
        self.assertEqual(stats.avg_time, 0)
        self.assertEqual(stats.min_time, float('inf'))
        self.assertEqual(stats.max_time, 0)

        keeper.enable()

        stats = keeper.get_stats('foo')
        snapshot_length = len(keeper.snapshot())
        self.assertEqual(snapshot_length, 1,
                         "Verify that getting a new stats with an enabled "
                         "keeper results in one time statistics. "
                         "Actual number = %d" % snapshot_length)

        self.assertEqual(stats.keeper, keeper)
        self.assertEqual(stats.name, 'foo')
        self.assertEqual(stats.count, 0)
        self.assertEqual(stats.avg_time, 0)
        self.assertEqual(stats.min_time, float('inf'))
        self.assertEqual(stats.max_time, 0)

        keeper.get_stats('foo')
        snapshot_length = len(keeper.snapshot())
        self.assertEqual(snapshot_length, 1,
                         "Verify that getting an existing stats with an "
                         "enabled keeper results in the same number of time "
                         "statistics. "
                         "Actual number = %d" % snapshot_length)

    def test_measure_enabled(self):
        """Test measuring time with enabled keeper."""

        keeper = TimeStatsKeeper()
        keeper.enable()

        # TimeStatsKeeper on Windows has only a precision of 1/60 sec
        duration = 1.6
        delta = duration / 10.0

        stats = keeper.get_stats('foo')
        dur = measure(stats, duration)

        for _, stats in keeper.snapshot():
            self.assertEqual(stats.count, 1)
            self.assertLess(time_abs_delta(stats.avg_time, dur), delta,
                            "avg time: actual: %f, expected: %f, delta: %f" %
                            (stats.avg_time, dur, delta))
            self.assertLess(time_abs_delta(stats.min_time, dur), delta,
                            "min time: actual: %f, expected: %f, delta: %f" %
                            (stats.min_time, dur, delta))
            self.assertLess(time_abs_delta(stats.max_time, dur), delta,
                            "max time: actual: %f, expected: %f, delta: %f" %
                            (stats.max_time, dur, delta))

        stats.reset()
        self.assertEqual(stats.count, 0)
        self.assertEqual(stats.avg_time, 0)
        self.assertEqual(stats.min_time, float('inf'))
        self.assertEqual(stats.max_time, 0)

    def test_measure_disabled(self):
        """Test measuring time with disabled keeper."""

        keeper = TimeStatsKeeper()

        duration = 0.2

        stats = keeper.get_stats('foo')
        self.assertEqual(stats.name, 'disabled')

        stats.begin()
        time.sleep(duration)
        stats.end()

        for _, stats in keeper.snapshot():
            self.assertEqual(stats.count, 0)
            self.assertEqual(stats.avg_time, 0)
            self.assertEqual(stats.min_time, float('inf'))
            self.assertEqual(stats.max_time, 0)

    def test_snapshot(self):
        """Test that snapshot() takes a stable snapshot."""

        keeper = TimeStatsKeeper()
        keeper.enable()

        duration = 0.2

        stats = keeper.get_stats('foo')

        # produce a first data item
        stats.begin()
        time.sleep(duration)
        stats.end()

        # take the snapshot
        snapshot = keeper.snapshot()

        # produce a second data item
        stats.begin()
        time.sleep(duration)
        stats.end()

        # verify that only the first data item is in the snapshot
        for _, snap_stats in snapshot:
            self.assertEqual(snap_stats.count, 1)

        # verify that both data items are in the original stats object
        self.assertEqual(stats.count, 2)

    def test_measure_avg_min_max(self):
        """Test measuring avg min max values."""

        keeper = TimeStatsKeeper()
        keeper.enable()

        # TimeStatsKeeper on Windows has only a precision of 1/60 sec
        durations = (0.6, 1.2, 1.5)
        delta = 0.08
        count = len(durations)

        stats = keeper.get_stats('foo')
        m_durations = []
        for duration in durations:
            m_durations.append(measure(stats, duration))

        min_dur = min(m_durations)
        max_dur = max(m_durations)
        avg_dur = sum(m_durations) / float(count)

        for _, stats in keeper.snapshot():
            self.assertEqual(stats.count, 3)
            self.assertLess(time_abs_delta(stats.avg_time, avg_dur), delta,
                            "avg time: actual: %f, expected: %f, delta: %f" %
                            (stats.avg_time, avg_dur, delta))
            self.assertLess(time_abs_delta(stats.min_time, min_dur), delta,
                            "min time: actual: %f, expected: %f, delta: %f" %
                            (stats.min_time, min_dur, delta))
            self.assertLess(time_abs_delta(stats.max_time, max_dur), delta,
                            "max time: actual: %f, expected: %f, delta: %f" %
                            (stats.max_time, max_dur, delta))

    def test_str_empty(self):
        """Test str() for an empty enabled keeper."""

        keeper = TimeStatsKeeper()
        keeper.enable()
        s = str(keeper)
        self.assertEqual(s, PRINT_HEADER)

    def test_str_disabled(self):
        """Test str() for a disabled keeper."""

        keeper = TimeStatsKeeper()
        s = str(keeper)
        self.assertEqual(s, PRINT_HEADER_DISABLED)


if __name__ == '__main__':
    unittest.main()



# Copyright 2016 IBM Corp. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
A utility class that handles HTTP methods against HMC URIs, based on the
faked HMC.

Note: At this point, the following HTTP methods needed by the zhmcclient
have not been implemented yet::

    POST     /api/partitions/([^/]+)/operations/scsi-dump
    POST     /api/partitions/([^/]+)/operations/psw-restart
    POST     /api/partitions/([^/]+)/operations/mount-iso-image
    POST     /api/partitions/([^/]+)/operations/unmount-iso-image
    POST     /api/partitions/([^/]+)/hbas/([^/]+)/operations/reassign-
               storage-adapter-port
    POST     /api/virtual-switches/([^/]+)/operations/get-connected-vnics
    POST     /api/cpcs/([^/]+)/operations/import-profiles
    POST     /api/cpcs/([^/]+)/operations/export-profiles
"""

from __future__ import absolute_import

import re

__all__ = ['UriHandler', 'HTTPError', 'URIS']


class HTTPError(Exception):

    def __init__(self, method, uri, http_status, reason, message):
        self.method = method
        self.uri = uri
        self.http_status = http_status
        self.reason = reason
        self.message = message

    def response(self):
        return {
            'request-method': self.method,
            'request-uri': self.uri,
            'http-status': self.http_status,
            'reason': self.reason,
            'message': self.message,
        }


class InvalidResourceError(HTTPError):

    def __init__(self, method, uri, handler_class=None):
        if handler_class is not None:
            handler_txt = "handler class %s" % handler_class.__name__
        else:
            handler_txt = "no handler class"
        super(InvalidResourceError, self).__init__(
            method, uri,
            http_status=404,
            reason=1,
            message="Unknown resource with URI: %s (%s)" % (uri, handler_txt))


class InvalidMethodError(HTTPError):

    def __init__(self, method, uri, handler_class=None):
        if handler_class is not None:
            handler_txt = "handler class %s" % handler_class.__name__
        else:
            handler_txt = "no handler class"
        super(InvalidMethodError, self).__init__(
            method, uri,
            http_status=404,
            reason=1,
            message="Invalid HTTP method %s on URI: %s %s" %
            (method, uri, handler_txt))


class CpcNotInDpmError(HTTPError):

    def __init__(self, method, uri, cpc):
        super(CpcNotInDpmError, self).__init__(
            method, uri,
            http_status=409,
            reason=5,
            message="CPC is not in DPM mode: %s" % cpc.uri)


class CpcInDpmError(HTTPError):

    def __init__(self, method, uri, cpc):
        super(CpcInDpmError, self).__init__(
            method, uri,
            http_status=409,
            reason=4,
            message="CPC is in DPM mode: %s" % cpc.uri)


class UriHandler(object):
    """
    Handle HTTP methods against a set of known URIs and invoke respective
    handlers.
    """

    def __init__(self, uris):
        self._uri_handlers = []  # tuple of (regexp-pattern, handler-name)
        for uri, handler_class in uris:
            uri_pattern = re.compile('^' + uri + '$')
            tup = (uri_pattern, handler_class)
            self._uri_handlers.append(tup)

    def handler(self, uri, method):
        for uri_pattern, handler_class in self._uri_handlers:
            m = uri_pattern.match(uri)
            if m:
                uri_parms = m.groups()
                return handler_class, uri_parms
        raise InvalidResourceError(method, uri)

    def get(self, hmc, uri, logon_required):
        handler_class, uri_parms = self.handler(uri, 'GET')
        if not getattr(handler_class, 'get', None):
            raise InvalidMethodError('GET', uri, handler_class)
        return handler_class.get(hmc, uri, uri_parms, logon_required)

    def post(self, hmc, uri, body, logon_required, wait_for_completion):
        handler_class, uri_parms = self.handler(uri, 'POST')
        if not getattr(handler_class, 'post', None):
            raise InvalidMethodError('POST', uri, handler_class)
        return handler_class.post(hmc, uri, uri_parms, body, logon_required,
                                  wait_for_completion)

    def delete(self, hmc, uri, logon_required):
        handler_class, uri_parms = self.handler(uri, 'DELETE')
        if not getattr(handler_class, 'delete', None):
            raise InvalidMethodError('DELETE', uri, handler_class)
        handler_class.delete(hmc, uri, uri_parms, logon_required)


class GenericGetPropertiesHandler(object):

    @staticmethod
    def get(hmc, uri, uri_parms, logon_required):
        """Operation: Get <resource> Properties."""
        try:
            resource = hmc.lookup_by_uri(uri)
        except KeyError:
            raise InvalidResourceError('GET', uri)
        return resource.properties


class GenericUpdatePropertiesHandler(object):

    @staticmethod
    def post(hmc, uri, uri_parms, body, logon_required, wait_for_completion):
        """Operation: Update <resource> Properties."""
        assert wait_for_completion is True  # async not supported yet
        try:
            resource = hmc.lookup_by_uri(uri)
        except KeyError:
            raise InvalidResourceError('GET', uri)
        resource.update(body)


class VersionHandler(object):

    @staticmethod
    def get(hmc, uri, uri_parms, logon_required):
        api_major, api_minor = hmc.api_version.split('.')
        return {
            'hmc-name': hmc.hmc_name,
            'hmc-version': hmc.hmc_version,
            'api-major-version': int(api_major),
            'api-minor-version': int(api_minor),
        }


class CpcsHandler(object):

    @staticmethod
    def get(hmc, uri, uri_parms, logon_required):
        """Operation: List CPCs."""
        result_cpcs = []
        for cpc in hmc.cpcs.list():
            result_cpc = {}
            for prop in cpc.properties:
                if prop in ('object-uri', 'name', 'status'):
                    result_cpc[prop] = cpc.properties[prop]
            result_cpcs.append(result_cpc)
        return {'cpcs': result_cpcs}


class CpcHandler(GenericGetPropertiesHandler,
                 GenericUpdatePropertiesHandler):
    pass


class CpcStartHandler(object):

    @staticmethod
    def post(hmc, uri, uri_parms, body, logon_required, wait_for_completion):
        """Operation: Start CPC (requires DPM mode)."""
        assert wait_for_completion is True  # async not supported yet
        cpc_oid = uri_parms[0]
        try:
            cpc = hmc.cpcs.lookup_by_oid(cpc_oid)
        except KeyError:
            raise InvalidResourceError('POST', uri)
        if not cpc.dpm_enabled:
            raise CpcNotInDpmError('POST', uri, cpc)
        cpc.properties['status'] = 'active'


class CpcStopHandler(object):

    @staticmethod
    def post(hmc, uri, uri_parms, body, logon_required, wait_for_completion):
        """Operation: Stop CPC (requires DPM mode)."""
        assert wait_for_completion is True  # async not supported yet
        cpc_oid = uri_parms[0]
        try:
            cpc = hmc.cpcs.lookup_by_oid(cpc_oid)
        except KeyError:
            raise InvalidResourceError('POST', uri)
        if not cpc.dpm_enabled:
            raise CpcNotInDpmError('POST', uri, cpc)
        cpc.properties['status'] = 'not-operating'


class CpcExportPortNamesListHandler(object):

    @staticmethod
    def post(hmc, uri, uri_parms, body, logon_required, wait_for_completion):
        """Operation: Export WWPN List (requires DPM mode)."""
        assert wait_for_completion is True  # this operation is always synchr.
        cpc_oid = uri_parms[0]
        try:
            cpc = hmc.cpcs.lookup_by_oid(cpc_oid)
        except KeyError:
            raise InvalidResourceError('POST', uri)
        if not cpc.dpm_enabled:
            raise CpcNotInDpmError('POST', uri, cpc)

        if body is None or 'partitions' not in body:
            raise HTTPError('POST', uri, 400,
                            149,  # TODO: Maybe use different reason?
                            "No 'partitions' property provided in request "
                            "body.")
        partition_uris = body['partitions']
        if len(partition_uris) == 0:
            raise HTTPError('POST', uri, 400, 149,
                            "'partitions' property provided in request "
                            "body is empty.")

        wwpn_list = []
        for partition_uri in partition_uris:
            partition = hmc.lookup_by_uri(partition_uri)
            partition_cpc = partition.manager.parent
            if partition_cpc.oid != cpc_oid:
                raise HTTPError('POST', uri, 400,
                                149,  # TODO: Maybe use different reason?
                                "Partition with object ID %s specified in "
                                "'partitions' property is not in CPC with "
                                "object ID %s." % (partition.oid, cpc_oid))
            partition_name = partition.properties.get('name', '')
            for hba in partition.hbas.list():
                port_uri = hba.properties['adapter-port-uri']
                port = hmc.lookup_by_uri(port_uri)
                adapter = port.manager.parent
                devno = hba.properties.get('device-number', '')
                wwpn = hba.properties.get('wwpn', '')
                wwpn_str = '%s,%s,%s,%s' % (partition_name, adapter.oid,
                                            devno, wwpn)
                wwpn_list.append(wwpn_str)
        return {
            'wwpn-list': wwpn_list
        }


class AdaptersHandler(object):

    @staticmethod
    def get(hmc, uri, uri_parms, logon_required):
        """Operation: List Adapters of a CPC."""
        cpc_oid = uri_parms[0]
        try:
            cpc = hmc.cpcs.lookup_by_oid(cpc_oid)
        except KeyError:
            raise InvalidResourceError('GET', uri)
        if not cpc.dpm_enabled:
            raise InvalidResourceError('GET', uri)  # in List: not found
        result_adapters = []
        for adapter in cpc.adapters.list():
            result_adapter = {}
            for prop in adapter.properties:
                if prop in ('object-uri', 'name', 'status'):
                    result_adapter[prop] = adapter.properties[prop]
            result_adapters.append(result_adapter)
        return {'adapters': result_adapters}


class AdapterHandler(GenericGetPropertiesHandler,
                     GenericUpdatePropertiesHandler):
    pass


class NetworkPortHandler(GenericGetPropertiesHandler,
                         GenericUpdatePropertiesHandler):
    pass


class StoragePortHandler(GenericGetPropertiesHandler,
                         GenericUpdatePropertiesHandler):
    pass


class PartitionsHandler(object):

    @staticmethod
    def get(hmc, uri, uri_parms, logon_required):
        """Operation: List Partitions of a CPC."""
        cpc_oid = uri_parms[0]
        try:
            cpc = hmc.cpcs.lookup_by_oid(cpc_oid)
        except KeyError:
            raise InvalidResourceError('GET', uri)
        if not cpc.dpm_enabled:
            raise InvalidResourceError('GET', uri)  # in List: not found
        result_partitions = []
        for partition in cpc.partitions.list():
            result_partition = {}
            for prop in partition.properties:
                if prop in ('object-uri', 'name', 'status'):
                    result_partition[prop] = partition.properties[prop]
            result_partitions.append(result_partition)
        return {'partitions': result_partitions}

    @staticmethod
    def post(hmc, uri, uri_parms, body, logon_required, wait_for_completion):
        """Operation: Create Partition."""
        assert wait_for_completion is True  # async not supported yet
        cpc_oid = uri_parms[0]
        try:
            cpc = hmc.cpcs.lookup_by_oid(cpc_oid)
        except KeyError:
            raise InvalidResourceError('POST', uri)
        if not cpc.dpm_enabled:
            raise CpcNotInDpmError('POST', uri, cpc)
        new_partition = cpc.partitions.add(body)
        return {'object-uri': new_partition.uri}


class PartitionHandler(GenericGetPropertiesHandler,
                       GenericUpdatePropertiesHandler):

    @staticmethod
    def delete(hmc, uri, uri_parms, logon_required):
        """Operation: Delete Partition."""
        try:
            partition = hmc.lookup_by_uri(uri)
        except KeyError:
            raise InvalidResourceError('DELETE', uri)
        cpc = partition.manager.parent
        if not cpc.dpm_enabled:
            raise CpcNotInDpmError('DELETE', uri, cpc)
        partition.manager.remove(partition.oid)


class PartitionStartHandler(object):

    @staticmethod
    def post(hmc, uri, uri_parms, body, logon_required, wait_for_completion):
        """Operation: Start Partition (requires DPM mode)."""
        assert wait_for_completion is True  # async not supported yet
        partition_uri = uri.split('/operations/')[0]
        try:
            partition = hmc.lookup_by_uri(partition_uri)
        except KeyError:
            raise InvalidResourceError('POST', uri)
        cpc = partition.manager.parent
        if not cpc.dpm_enabled:
            raise CpcNotInDpmError('POST', uri, cpc)
        partition.properties['status'] = 'active'


class PartitionStopHandler(object):

    @staticmethod
    def post(hmc, uri, uri_parms, body, logon_required, wait_for_completion):
        """Operation: Start Partition (requires DPM mode)."""
        assert wait_for_completion is True  # async not supported yet
        partition_uri = uri.split('/operations/')[0]
        try:
            partition = hmc.lookup_by_uri(partition_uri)
        except KeyError:
            raise InvalidResourceError('POST', uri)
        cpc = partition.manager.parent
        if not cpc.dpm_enabled:
            raise CpcNotInDpmError('POST', uri, cpc)
        partition.properties['status'] = 'stopped'


class HbasHandler(object):

    @staticmethod
    def post(hmc, uri, uri_parms, body, logon_required, wait_for_completion):
        """Operation: Create HBA."""
        assert wait_for_completion is True  # async not supported yet
        partition_uri = re.sub('/hbas$', '', uri)
        try:
            partition = hmc.lookup_by_uri(partition_uri)
        except KeyError:
            raise InvalidResourceError('POST', uri)
        cpc = partition.manager.parent
        if not cpc.dpm_enabled:
            raise CpcNotInDpmError('POST', uri, cpc)
        new_hba = partition.hbas.add(body)
        return {'element-uri': new_hba.uri}


class HbaHandler(GenericGetPropertiesHandler,
                 GenericUpdatePropertiesHandler):

    @staticmethod
    def delete(hmc, uri, uri_parms, logon_required):
        """Operation: Delete HBA."""
        try:
            hba = hmc.lookup_by_uri(uri)
        except KeyError:
            raise InvalidResourceError('DELETE', uri)
        partition = hba.manager.parent
        cpc = partition.manager.parent
        if not cpc.dpm_enabled:
            raise CpcNotInDpmError('DELETE', uri, cpc)
        partition.hbas.remove(hba.oid)


class NicsHandler(object):

    @staticmethod
    def post(hmc, uri, uri_parms, body, logon_required, wait_for_completion):
        """Operation: Create NIC."""
        assert wait_for_completion is True  # async not supported yet
        partition_uri = re.sub('/nics$', '', uri)
        try:
            partition = hmc.lookup_by_uri(partition_uri)
        except KeyError:
            raise InvalidResourceError('POST', uri)
        cpc = partition.manager.parent
        if not cpc.dpm_enabled:
            raise CpcNotInDpmError('POST', uri, cpc)
        new_nic = partition.nics.add(body)
        return {'element-uri': new_nic.uri}


class NicHandler(GenericGetPropertiesHandler,
                 GenericUpdatePropertiesHandler):

    @staticmethod
    def delete(hmc, uri, uri_parms, logon_required):
        """Operation: Delete NIC."""
        try:
            nic = hmc.lookup_by_uri(uri)
        except KeyError:
            raise InvalidResourceError('DELETE', uri)
        partition = nic.manager.parent
        cpc = partition.manager.parent
        if not cpc.dpm_enabled:
            raise CpcNotInDpmError('DELETE', uri, cpc)
        partition.nics.remove(nic.oid)


class VirtualFunctionsHandler(object):

    @staticmethod
    def post(hmc, uri, uri_parms, body, logon_required, wait_for_completion):
        """Operation: Create Virtual Function"""
        assert wait_for_completion is True  # async not supported yet
        partition_uri = re.sub('/virtual-functions$', '', uri)
        try:
            partition = hmc.lookup_by_uri(partition_uri)
        except KeyError:
            raise InvalidResourceError('POST', uri)
        cpc = partition.manager.parent
        if not cpc.dpm_enabled:
            raise CpcNotInDpmError('POST', uri, cpc)
        new_vf = partition.virtual_functions.add(body)
        return {'element-uri': new_vf.uri}


class VirtualFunctionHandler(GenericGetPropertiesHandler,
                             GenericUpdatePropertiesHandler):

    @staticmethod
    def delete(hmc, uri, uri_parms, logon_required):
        """Operation: Delete Virtual Function."""
        try:
            vf = hmc.lookup_by_uri(uri)
        except KeyError:
            raise InvalidResourceError('DELETE', uri)
        partition = vf.manager.parent
        cpc = partition.manager.parent
        if not cpc.dpm_enabled:
            raise CpcNotInDpmError('DELETE', uri, cpc)
        partition.virtual_functions.remove(vf.oid)


class VirtualSwitchesHandler(object):

    @staticmethod
    def get(hmc, uri, uri_parms, logon_required):
        """Operation: List Virtual Switches of a CPC."""
        cpc_oid = uri_parms[0]
        try:
            cpc = hmc.cpcs.lookup_by_oid(cpc_oid)
        except KeyError:
            raise InvalidResourceError('GET', uri)
        if not cpc.dpm_enabled:
            raise InvalidResourceError('GET', uri)  # in List: not found
        result_vswitches = []
        for vswitch in cpc.virtual_switches.list():
            result_vswitch = {}
            for prop in vswitch.properties:
                if prop in ('object-uri', 'name', 'type'):
                    result_vswitch[prop] = vswitch.properties[prop]
            result_vswitches.append(result_vswitch)
        return {'virtual-switches': result_vswitches}


class VirtualSwitchHandler(GenericGetPropertiesHandler,
                           GenericUpdatePropertiesHandler):
    pass


class LparsHandler(object):

    @staticmethod
    def get(hmc, uri, uri_parms, logon_required):
        """Operation: List Logical Partitions of CPC."""
        cpc_oid = uri_parms[0]
        try:
            cpc = hmc.cpcs.lookup_by_oid(cpc_oid)
        except KeyError:
            raise InvalidResourceError('GET', uri)
        if cpc.dpm_enabled:
            raise InvalidResourceError('GET', uri)  # in List: not found
        result_lpars = []
        for lpar in cpc.lpars.list():
            result_lpar = {}
            for prop in lpar.properties:
                if prop in ('object-uri', 'name', 'status'):
                    result_lpar[prop] = lpar.properties[prop]
            result_lpars.append(result_lpar)
        return {'logical-partitions': result_lpars}


class LparHandler(GenericGetPropertiesHandler,
                  GenericUpdatePropertiesHandler):
    pass


class LparActivateHandler(object):

    @staticmethod
    def post(hmc, uri, uri_parms, body, logon_required, wait_for_completion):
        """Operation: Activate Logical Partition (requires classic mode)."""
        assert wait_for_completion is True  # async not supported yet
        lpar_uri = uri.split('/operations/')[0]
        try:
            lpar = hmc.lookup_by_uri(lpar_uri)
        except KeyError:
            raise InvalidResourceError('POST', uri)
        cpc = lpar.manager.parent
        if cpc.dpm_enabled:
            raise CpcInDpmError('POST', uri, cpc)
        lpar.properties['status'] = 'not-operating'


class LparDeactivateHandler(object):

    @staticmethod
    def post(hmc, uri, uri_parms, body, logon_required, wait_for_completion):
        """Operation: Deactivate Logical Partition (requires classic mode)."""
        assert wait_for_completion is True  # async not supported yet
        lpar_uri = uri.split('/operations/')[0]
        try:
            lpar = hmc.lookup_by_uri(lpar_uri)
        except KeyError:
            raise InvalidResourceError('POST', uri)
        cpc = lpar.manager.parent
        if cpc.dpm_enabled:
            raise CpcInDpmError('POST', uri, cpc)
        lpar.properties['status'] = 'not-activated'


class LparLoadHandler(object):

    @staticmethod
    def post(hmc, uri, uri_parms, body, logon_required, wait_for_completion):
        """Operation: Load Logical Partition (requires classic mode)."""
        assert wait_for_completion is True  # async not supported yet
        lpar_uri = uri.split('/operations/')[0]
        try:
            lpar = hmc.lookup_by_uri(lpar_uri)
        except KeyError:
            raise InvalidResourceError('POST', uri)
        cpc = lpar.manager.parent
        if cpc.dpm_enabled:
            raise CpcInDpmError('POST', uri, cpc)
        lpar.properties['status'] = 'operating'


class ResetActProfilesHandler(object):

    @staticmethod
    def get(hmc, uri, uri_parms, logon_required):
        """Operation: List Reset Activation Profiles."""
        cpc_oid = uri_parms[0]
        try:
            cpc = hmc.cpcs.lookup_by_oid(cpc_oid)
        except KeyError:
            raise InvalidResourceError('GET', uri)
        if cpc.dpm_enabled:
            raise InvalidResourceError('GET', uri)  # in List: not found
        result_profiles = []
        for profile in cpc.reset_activation_profiles.list():
            result_profile = {}
            for prop in profile.properties:
                if prop in ('element-uri', 'name'):
                    result_profile[prop] = profile.properties[prop]
            result_profiles.append(result_profile)
        return {'reset-activation-profiles': result_profiles}


class ResetActProfileHandler(GenericGetPropertiesHandler,
                             GenericUpdatePropertiesHandler):
    pass


class ImageActProfilesHandler(object):

    @staticmethod
    def get(hmc, uri, uri_parms, logon_required):
        """Operation: List Image Activation Profiles."""
        cpc_oid = uri_parms[0]
        try:
            cpc = hmc.cpcs.lookup_by_oid(cpc_oid)
        except KeyError:
            raise InvalidResourceError('GET', uri)
        if cpc.dpm_enabled:
            raise InvalidResourceError('GET', uri)  # in List: not found
        result_profiles = []
        for profile in cpc.image_activation_profiles.list():
            result_profile = {}
            for prop in profile.properties:
                if prop in ('element-uri', 'name'):
                    result_profile[prop] = profile.properties[prop]
            result_profiles.append(result_profile)
        return {'image-activation-profiles': result_profiles}


class ImageActProfileHandler(GenericGetPropertiesHandler,
                             GenericUpdatePropertiesHandler):
    pass


class LoadActProfilesHandler(object):

    @staticmethod
    def get(hmc, uri, uri_parms, logon_required):
        """Operation: List Load Activation Profiles."""
        cpc_oid = uri_parms[0]
        try:
            cpc = hmc.cpcs.lookup_by_oid(cpc_oid)
        except KeyError:
            raise InvalidResourceError('GET', uri)
        if cpc.dpm_enabled:
            raise InvalidResourceError('GET', uri)  # in List: not found
        result_profiles = []
        for profile in cpc.load_activation_profiles.list():
            result_profile = {}
            for prop in profile.properties:
                if prop in ('element-uri', 'name'):
                    result_profile[prop] = profile.properties[prop]
            result_profiles.append(result_profile)
        return {'load-activation-profiles': result_profiles}


class LoadActProfileHandler(GenericGetPropertiesHandler,
                            GenericUpdatePropertiesHandler):
    pass


# URIs to be handled
URIS = (

    # In all modes:

    ('/api/version', VersionHandler),

    ('/api/cpcs', CpcsHandler),
    ('/api/cpcs/([^/]+)', CpcHandler),

    # Only in DPM mode:

    ('/api/cpcs/([^/]+)/operations/start', CpcStartHandler),
    ('/api/cpcs/([^/]+)/operations/stop', CpcStopHandler),
    ('/api/cpcs/([^/]+)/operations/export-port-names-list',
     CpcExportPortNamesListHandler),

    ('/api/cpcs/([^/]+)/adapters', AdaptersHandler),
    ('/api/adapters/([^/]+)', AdapterHandler),

    ('/api/adapters/([^/]+)/network-ports/([^/]+)', NetworkPortHandler),

    ('/api/adapters/([^/]+)/storage-ports/([^/]+)', StoragePortHandler),

    ('/api/cpcs/([^/]+)/partitions', PartitionsHandler),
    ('/api/partitions/([^/]+)', PartitionHandler),
    ('/api/partitions/([^/]+)/operations/start', PartitionStartHandler),
    ('/api/partitions/([^/]+)/operations/stop', PartitionStopHandler),
    # ('/api/partitions/([^/]+)/operations/scsi-dump',
    #  PartitionScsiDumpHandler),
    # ('/api/partitions/([^/]+)/operations/psw-restart',
    #  PartitionPswRestartHandler),
    # ('/api/partitions/([^/]+)/operations/mount-iso-image',
    #  PartitionMountIsoImageHandler),
    # ('/api/partitions/([^/]+)/operations/unmount-iso-image',
    #  PartitionUnmountIsoImageHandler),

    ('/api/partitions/([^/]+)/hbas', HbasHandler),
    ('/api/partitions/([^/]+)/hbas/([^/]+)', HbaHandler),
    # ('/api/partitions/([^/]+)/hbas/([^/]+)/operations/'\
    #  'reassign-storage-adapter-port', HbaReassignPortHandler),

    ('/api/partitions/([^/]+)/nics', NicsHandler),
    ('/api/partitions/([^/]+)/nics/([^/]+)', NicHandler),

    ('/api/partitions/([^/]+)/virtual-functions', VirtualFunctionsHandler),
    ('/api/partitions/([^/]+)/virtual-functions/([^/]+)',
     VirtualFunctionHandler),

    ('/api/cpcs/([^/]+)/virtual-switches', VirtualSwitchesHandler),
    ('/api/virtual-switches/([^/]+)', VirtualSwitchHandler),
    # ('/api/virtual-switches/([^/]+)/operations/get-connected-vnics',
    #  VirtualSwitchGetVnicsHandler),

    # Only in classic (or ensemble) mode:

    # ('/api/cpcs/([^/]+)/operations/import-profiles',
    #  CpcImportProfilesHandler),
    # ('/api/cpcs/([^/]+)/operations/export-profiles',
    #  CpcExportProfilesHandler),

    ('/api/cpcs/([^/]+)/logical-partitions', LparsHandler),
    ('/api/logical-partitions/([^/]+)', LparHandler),
    ('/api/logical-partitions/([^/]+)/operations/activate',
     LparActivateHandler),
    ('/api/logical-partitions/([^/]+)/operations/deactivate',
     LparDeactivateHandler),
    ('/api/logical-partitions/([^/]+)/operations/load', LparLoadHandler),

    ('/api/cpcs/([^/]+)/reset-activation-profiles', ResetActProfilesHandler),
    ('/api/cpcs/([^/]+)/reset-activation-profiles/([^/]+)',
     ResetActProfileHandler),

    ('/api/cpcs/([^/]+)/image-activation-profiles', ImageActProfilesHandler),
    ('/api/cpcs/([^/]+)/image-activation-profiles/([^/]+)',
     ImageActProfileHandler),

    ('/api/cpcs/([^/]+)/load-activation-profiles', LoadActProfilesHandler),
    ('/api/cpcs/([^/]+)/load-activation-profiles/([^/]+)',
     LoadActProfileHandler),
)



# Copyright 2016 IBM Corp. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
This package supports logging using the standard Python :mod:`py:logging`
module. The logging support provides two :class:`~py:logging.Logger` objects:

* 'zhmcclient.api' for user-issued calls to zhmcclient API functions, at the
  debug level. Internal calls to API functions are not logged.

* 'zhmcclient.hmc' for interactions between zhmcclient and the HMC, at the
  debug level.

In addition, there are loggers for each module with the module name, for
situations like errors or warnings.

For HMC operations and API calls that contain the HMC password, the password
is hidden in the log message by replacing it with a few '*' characters.

All these loggers have a null-handler (see :class:`~py:logging.NullHandler`)
and have no log formatter (see :class:`~py:logging.Formatter`).

As a result, the loggers are silent by default. If you want to turn on logging,
add a log handler (see :meth:`~py:logging.Logger.addHandler`, and
:mod:`py:logging.handlers` for the handlers included with Python) and set the
log level (see :meth:`~py:logging.Logger.setLevel`, and :ref:`py:levels` for
the defined levels).

If you want to change the default log message format, use
:meth:`~py:logging.Handler.setFormatter`. Its ``form`` parameter is a format
string with %-style placeholders for the log record attributes (see Python
section :ref:`py:logrecord-attributes`).

Examples:

* To output the log records for all HMC operations to ``stdout`` in a
  particular format, do this:

  ::

      import logging

      handler = logging.StreamHandler()
      format_string = '%(asctime)s - %(name)s - %(levelname)s - %(message)s'
      handler.setFormatter(logging.Formatter(format_string))
      logger = getLogger('zhmcclient.hmc')
      logger.addHandler(handler)
      logger.setLevel(logging.DEBUG)

* This example uses the :func:`~py:logging.basicConfig` convenience function
  that sets the same format and level as in the previous example, but for the
  root logger. Therefore, it will output all log records, not just from this
  package:

  ::

      import logging

      format_string = '%(asctime)s - %(name)s - %(levelname)s - %(message)s'
      logging.basicConfig(format=format_string, level=logging.DEBUG)
"""

import logging
import inspect
from decorator import decorate

from ._constants import API_LOGGER_NAME


def get_logger(name):
    """
    Return a :class:`~py:logging.Logger` object with the specified name.

    A :class:`~py:logging.NullHandler` handler is added to the logger if it
    does not have any handlers yet. This prevents the propagation of log
    requests up the Python logger hierarchy, and therefore causes this package
    to be silent by default.
    """
    logger = logging.getLogger(name)
    if not logger.handlers:
        logger.addHandler(logging.NullHandler())
    return logger


def logged_api_call(func):
    """
    Function decorator that causes the decorated API function or method to log
    calls to itself to a logger.

    The logger's name is the dotted module name of the module defining the
    decorated function (e.g. 'zhmcclient._cpc').

    Parameters:

      func (function object): The original function being decorated.

    Returns:

      function object: The function wrappering the original function being
        decorated.
    """

    # Note that in this decorator function, we are in a module loading context,
    # where the decorated functions are being defined. When this decorator
    # function is called, its call stack represents the definition of the
    # decorated functions. Not all global definitions in the module have been
    # defined yet, and methods of classes that are decorated with this
    # decorator are still functions at this point (and not yet methods).

    module = inspect.getmodule(func)
    if not getattr(module, '__name__', None):
        raise TypeError("The @logged_api_call decorator must be used on a "
                        "function or method (and not on top of the @property "
                        "decorator)")
    if not inspect.isfunction(func):
        raise TypeError("The @logged_api_call decorator must be used on a "
                        "function or method ")

    try:
        # We avoid the use of inspect.getouterframes() because it is slow,
        # and use the pointers up the stack frame, instead.

        this_frame = inspect.currentframe()  # this decorator function here
        apifunc_frame = this_frame.f_back  # the decorated API function

        apifunc_owner = inspect.getframeinfo(apifunc_frame)[2]

    finally:
        # Recommended way to deal with frame objects to avoid ref cycles
        del this_frame
        del apifunc_frame

    # TODO: For inner functions, show all outer levels instead of just one.

    if apifunc_owner == '<module>':
        # The decorated API function is defined globally (at module level)
        apifunc_str = '{func}()'.format(func=func.__name__)
    else:
        # The decorated API function is defined in a class or in a function
        apifunc_str = '{owner}.{func}()'.format(owner=apifunc_owner,
                                                func=func.__name__)

    logger = get_logger(API_LOGGER_NAME)

    def log_api_call(func, *args, **kwargs):
        """
        Log entry to and exit from the decorated function, at the debug level.

        Note that this wrapper function is called every time the decorated
        function/method is called, but that the log message only needs to be
        constructed when logging for this logger and for this log level is
        turned on. Therefore, we do as much as possible in the decorator
        function, plus we use %-formatting and lazy interpolation provided by
        the log functions, in order to save resources in this function here.

        Parameters:

          func (function object): The decorated function.

          *args: Any positional arguments for the decorated function.

          **kwargs: Any keyword arguments for the decorated function.
        """

        # Note that in this function, we are in the context where the
        # decorated function is actually called.

        try:
            # We avoid the use of inspect.getouterframes() because it is slow,
            # and use the pointers up the stack frame, instead.

            this_frame = inspect.currentframe()  # this function here
            apifunc_frame = this_frame.f_back  # the decorated API function
            apicaller_frame = apifunc_frame.f_back  # caller of API function

            apicaller_module = inspect.getmodule(apicaller_frame)
            apicaller_module_name = apicaller_module.__name__
        finally:
            # Recommended way to deal with frame objects to avoid ref cycles
            del this_frame
            del apifunc_frame
            del apicaller_frame
            del apicaller_module

        # Log only if the caller is not from our package
        log_it = (apicaller_module_name.split('.')[0] != 'zhmcclient')

        if log_it:
            logger.debug("==> %s, args: %.500r, kwargs: %.500r",
                         apifunc_str, args, kwargs)
        result = func(*args, **kwargs)
        if log_it:
            logger.debug("<== %s, result: %.1000r", apifunc_str, result)
        return result

    return decorate(func, log_api_call)



# Copyright 2016 IBM Corp. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import absolute_import

import json
from collections import OrderedDict
import click
import click_spinner
from tabulate import tabulate

import zhmcclient

# Display of options in usage line
GENERAL_OPTIONS_METAVAR = '[GENERAL-OPTIONS]'
COMMAND_OPTIONS_METAVAR = '[COMMAND-OPTIONS]'

# File path of history file for interactive mode.
# If the file name starts with tilde (which is handled by the shell, not by
# the file system), it is properly expanded.
REPL_HISTORY_FILE = '~/.zhmc_history'

REPL_PROMPT = u'zhmc> '  # Must be Unicode

TABLE_FORMATS = ['table', 'plain', 'simple', 'psql', 'rst', 'mediawiki',
                 'html', 'latex']

LOG_DESTINATIONS = ['stderr', 'syslog', 'none']

LOG_LEVELS = ['error', 'warning', 'info', 'debug']

LOG_COMPONENTS = ['api', 'hmc', 'all']

SYSLOG_FACILITIES = ['user', 'local0', 'local1', 'local2', 'local3', 'local4',
                     'local5', 'local6', 'local7']


def abort_if_false(ctx, param, value):
    # pylint: disable=unused-argument
    if not value:
        raise click.ClickException("Aborted.")


class InvalidOutputFormatError(click.ClickException):
    """
    Exception indicating an invalid output format for zhmc.
    """

    def __init__(self, output_format):
        msg = "Invalid output format: {of}".format(of=output_format)
        super(InvalidOutputFormatError, self).__init__(msg)


class CmdContext(object):

    def __init__(self, host, userid, password, output_format, timestats,
                 session_id, get_password):
        self._host = host
        self._userid = userid
        self._password = password
        self._output_format = output_format
        self._timestats = timestats
        self._session_id = session_id
        self._get_password = get_password
        self._session = None
        self._spinner = click_spinner.Spinner()

    def __repr__(self):
        ret = "CmdContext(at 0x%08x, host=%r, userid=%r, password=%r, " \
            "output_format=%r, session_id=%r, session=%r, ...)" % \
            (id(self), self._host, self._userid,
             '...' if self._password else None, self._output_format,
             self._session_id, self._session)
        return ret

    @property
    def host(self):
        """
        :term:`string`: Hostname or IP address of the HMC.
        """
        return self._host

    @property
    def userid(self):
        """
        :term:`string`: Userid on the HMC.
        """
        return self._userid

    @property
    def output_format(self):
        """
        :term:`string`: Output format to be used.
        """
        return self._output_format

    @property
    def timestats(self):
        """
        bool: Indicates whether time statistics should be printed.
        """
        return self._timestats

    @property
    def session_id(self):
        """
        :term:`string`: Session-id to be used instead of logging on, or `None`.
        """
        return self._session_id

    @property
    def get_password(self):
        """
        :term:`callable`: Password retrieval function, or `None`.
        """
        return self._get_password

    @property
    def session(self):
        """
        :class:`requests.Session` object once logged on, or `None`.
        """
        return self._session

    @property
    def spinner(self):
        """
        :class:`~click_spinner.Spinner` object.

        Since click_spinner 0.1.5, the Spinner object takes care of suppressing
        the spinner when not on a tty, and is able to suspend/resume the
        spinner via its stop() and start() methods.
        """
        return self._spinner

    def execute_cmd(self, cmd):
        if self._session is None:
            if self._host is None:
                raise click.ClickException("No HMC host provided")
            self._session = zhmcclient.Session(
                self._host, self._userid, self._password,
                session_id=self._session_id, get_password=self._get_password)
        if self.timestats:
            self._session.time_stats_keeper.enable()
        self.spinner.start()
        try:
            cmd()
        finally:
            self.spinner.stop()
            if self._session.time_stats_keeper.enabled:
                click.echo(self._session.time_stats_keeper)


def original_options(options):
    """
    Return the input options with their original names.

    This is used to undo the name change the click package applies
    automatically before passing the options to the function that was decorated
    with 'click.option()'. The original names are needed in case there is
    special processing of the options on top of 'options_to_properties()'.

    The original names are constructed by replacing any underscores '_' with
    hyphens '-'. This approach may not be perfect in general, but it works for
    the zhmc CLI because the original option names do not have any underscores.

    Parameters:

      options (dict): The click options dictionary as passed to the decorated
        function by click (key: option name as changed by click, value: option
        value).

    Returns:

      dict: Options with their original names.
    """
    org_options = {}
    for name, value in options.iteritems():
        org_name = name.replace('_', '-')
        org_options[org_name] = value
    return org_options


def options_to_properties(options, name_map=None):
    """
    Convert click options into HMC resource properties.

    The click option names in input parameters to this function are the
    original option names (e.g. as produced by `original_options()`.

    Options with a value of `None` are not added to the returned resource
    properties.

    If a name mapping dictionary is specified, the option names are mapped
    using that dictionary. If an option name is mapped to `None`, it is not
    going to be added to the set of returned resource properties.

    Parameters:

      options (dict): The options dictionary (key: original option name,
        value: option value).

      name_map (dict): `None` or name mapping dictionary (key: original
        option name, value: property name, or `None` to not add this option to
        the returned properties).

    Returns:

      dict: Resource properties (key: property name, value: option value)
    """
    properties = {}
    for name, value in options.iteritems():
        if value is None:
            continue
        if name_map:
            name = name_map.get(name, name)
        if name is not None:
            properties[name] = value
    return properties


def print_properties(properties, output_format, skip_list=None):
    """
    Print properties in the desired output format.
    """
    if output_format in TABLE_FORMATS:
        if output_format == 'table':
            output_format = 'psql'
        print_properties_as_table(properties, output_format, skip_list)
    elif output_format == 'json':
        print_properties_as_json(properties)
    else:
        raise InvalidOutputFormatError(output_format)


def print_resources(resources, output_format, show_list=None):
    """
    Print the properties of a list of resources in the desired output format.
    """
    if output_format in TABLE_FORMATS:
        if output_format == 'table':
            output_format = 'psql'
        print_resources_as_table(resources, output_format, show_list)
    elif output_format == 'json':
        print_resources_as_json(resources, show_list)
    else:
        raise InvalidOutputFormatError(output_format)


def print_properties_as_table(properties, table_format, skip_list=None):
    """
    Print properties in tabular output format.

    The order of rows is ascending by property name.

    Parameters:

      properties (dict): The properties.

      table_format: Supported table formats are:
         - "table" -> same like "psql"
         - "plain"
         - "simple"
         - "psql"
         - "rst"
         - "mediawiki"
         - "html"
         - "latex"

      skip_list (iterable of string): The property names to be skipped.
        If `None`, all properties are shown.
    """
    additional_skip_list = (
        '@@implementation-errors',
    )
    table = list()
    sorted_fields = sorted(properties)
    for field in sorted_fields:
        if skip_list and field in skip_list or field in additional_skip_list:
            continue
        value = properties[field]
        table.append((field, value))
    headers = ['Field Name', 'Value']
    click.echo(tabulate(table, headers, tablefmt=table_format))


def print_resources_as_table(resources, table_format, show_list=None):
    """
    Print resources in tabular output format.

    Parameters:

      resources (iterable of BaseResource):
        The resources.

      table_format: Supported table formats are:
         - "table" -> same like "psql"
         - "plain"
         - "simple"
         - "psql"
         - "rst"
         - "mediawiki"
         - "html"
         - "latex"

      show_list (iterable of string):
        The property names to be shown. If a property is not in the resource
        object, it will be retrieved from the HMC. This iterable also defines
        the order of columns in the table, from left to right in iteration
        order.

        If `None`, all properties in the resource objects are shown, and their
        column order is ascending by property name.
    """
    table = list()
    for i, resource in enumerate(resources):
        properties = OrderedDict()
        if show_list:
            for name in show_list:
                # By using prop(), the resource with the full set of
                # properties will be retrieved, if a desired property is not
                # yet in the resource object
                properties[name] = resource.prop(name)
        else:
            for name in sorted(resource.properties.keys()):
                properties[name] = resource.prop(name)
        if i == 0:
            headers = properties.keys()
        row = properties.values()
        table.append(row)
    if not table:
        click.echo("No resources.")
    else:
        sorted_table = sorted(table, key=lambda row: row[0])
        click.echo(tabulate(sorted_table, headers, tablefmt=table_format))


def print_properties_as_json(properties):
    """
    Print properties in JSON output format.

    Parameters:

      properties (dict): The properties.
    """
    json_str = json.dumps(properties)
    click.echo(json_str)


def print_resources_as_json(resources, show_list=None):
    """
    Print resources in JSON output format.

    Parameters:

      resources (iterable of BaseResource):
        The resources.

      show_list (iterable of string):
        The property names to be shown. If a property is not in a resource
        object, it will be retrieved from the HMC.

        If `None`, all properties in the input resource objects are shown.
    """
    json_obj = list()
    for i, resource in enumerate(resources):
        if show_list:
            properties = OrderedDict()
            for name in show_list:
                # By using prop(), the resource with the full set of
                # properties will be retrieved, if a desired property is not
                # yet in the resource object
                properties[name] = resource.prop(name, None)
        else:
            properties = resource.properties
        json_obj.append(properties)
    json_str = json.dumps(json_obj)
    click.echo(json_str)



# Copyright 2016 IBM Corp. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Utility module for the Jupyter notebooks used as tutorials.
"""

import getpass
import requests
import six

import zhmcclient

requests.packages.urllib3.disable_warnings()

USERID = None
PASSWORD = None


def make_client(zhmc, userid=None, password=None):
    """
    Create a `Session` object for the specified HMC and log that on. Create a
    `Client` object using that `Session` object, and return it.

    If no userid and password are specified, and if no previous call to this
    method was made, userid and password are interactively inquired.
    Userid and password are saved in module-global variables for future calls
    to this method.
    """

    global USERID, PASSWORD  # pylint: disable=global-statement

    USERID = userid or USERID or \
        six.input('Enter userid for HMC {}: '.format(zhmc))
    PASSWORD = password or PASSWORD or \
        getpass.getpass('Enter password for {}: '.format(USERID))

    session = zhmcclient.Session(zhmc, USERID, PASSWORD)
    session.logon()
    client = zhmcclient.Client(session)
    print('Established logged-on session with HMC {} using userid {}'.
          format(zhmc, USERID))
    return client



#!/usr/bin/env python
# Copyright 2016 IBM Corp. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Example shows lifecycle (Create-Read-Update-Delete) of a Partition.
"""

import sys
import logging
import yaml
import json
import requests.packages.urllib3
from pprint import pprint

import zhmcclient

requests.packages.urllib3.disable_warnings()

if len(sys.argv) != 2:
    print("Usage: %s hmccreds.yaml" % sys.argv[0])
    sys.exit(2)
hmccreds_file = sys.argv[1]

with open(hmccreds_file, 'r') as fp:
    hmccreds = yaml.load(fp)

examples = hmccreds.get("examples", None)
if examples is None:
    print("examples not found in credentials file %s" % \
          (hmccreds_file))
    sys.exit(1)

partition_lifecycle = examples.get("partition_lifecycle", None)
if partition_lifecycle is None:
    print("partition_lifecycle not found in credentials file %s" % \
          (hmccreds_file))
    sys.exit(1)

loglevel = partition_lifecycle.get("loglevel", None)
if loglevel is not None:
    level = getattr(logging, loglevel.upper(), None)
    if level is None:
        print("Invalid value for loglevel in credentials file %s: %s" % \
              (hmccreds_file, loglevel))
        sys.exit(1)
    logging.basicConfig(level=level)

hmc = partition_lifecycle["hmc"]
cpcname = partition_lifecycle["cpcname"]
partname = partition_lifecycle["partname"]

cred = hmccreds.get(hmc, None)
if cred is None:
    print("Credentials for HMC %s not found in credentials file %s" % \
          (hmc, hmccreds_file))
    sys.exit(1)

userid = cred["userid"]
password = cred["password"]

print(__doc__)

print("Using HMC %s with userid %s ..." % (hmc, userid))
session = zhmcclient.Session(hmc, userid, password)
cl = zhmcclient.Client(session)

timestats = partition_lifecycle.get("timestats", False)
if timestats:
    session.time_stats_keeper.enable()

print("Listing CPCs ...")
cpcs = cl.cpcs.list()
for cpc in cpcs:
    print(cpc)

print("Finding CPC by name=%s ..." % cpcname)
try:
    cpc = cl.cpcs.find(name=cpcname)
except zhmcclient.NotFound:
    print("Could not find CPC %s on HMC %s" % (cpcname, hmc))
    sys.exit(1)

print("Checking if DPM is enabled on CPC %s..." % cpc.name)
if not cpc.dpm_enabled:
    print("CPC %s is not in DPM mode." % cpc.name)
    sys.exit(1)
print("CPC %s is in DPM mode." % cpc.name)

print("Finding Partition by name=%s on CPC %s ..." % (partname, cpc.name))
try:
    partition = cpc.partitions.find(name=partname)
except zhmcclient.NotFound:
    print("Partition %s does not exist yet" % partname)
else:
    print("Partition %s already exists - cleaning it up" % partition.name)
    status = partition.get_property('status')
    print("Partition %s status: %s" % (partition.name, status))
    if status == 'active':
        print("Stopping Partition %s ..." % partition.name)
        partition.stop()
    print("Deleting Partition %s ..." % partition.name)
    partition.delete()

properties = {
     'name': partname,
     'description': 'Original partition description.',
     'cp-processors': 2,
     'initial-memory': 1024,
     'maximum-memory': 2048,
     'processor-mode': 'shared',
     'boot-device': 'test-operating-system'
}
print("Creating a new Partition %s on CPC %s with following properties ..."
      % (partname, cpcname))
pprint(properties)
new_partition = cpc.partitions.create(properties)
print("New Partition %s created at: %s" %
      (new_partition.name, new_partition.uri))

print("Starting Partition %s ..." % new_partition.name)
new_partition.start()

print("Description of Partition %s: %s"
    % (new_partition.name, new_partition.get_property('description')))

new_description = "Updated partition description."
print("Updating partition description to: %s" % new_description)
updated_properties = dict()
updated_properties["description"] = new_description
new_partition.update_properties(updated_properties)

print("Refreshing properties of Partition %s ..." % new_partition.name)
new_partition.pull_full_properties()
print("Description of Partition %s: %s"
    % (new_partition.name, new_partition.get_property('description')))

print("Logging off ...")
session.logoff()

if timestats:
    print(session.time_stats_keeper)

print("Done.")



#!/usr/bin/env python
# Copyright 2016 IBM Corp. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Unit tests for _manager module.
"""

from __future__ import absolute_import, print_function

import unittest

from zhmcclient._manager import BaseManager
from zhmcclient._resource import BaseResource


class MyResource(BaseResource):
    def __init__(self, manager, uri, name=None, properties=None):
        super(MyResource, self).__init__(manager, uri, name, properties)


class MyManager(BaseManager):
    def __init__(self, parent=None):
        super(MyManager, self).__init__(
            resource_class=MyResource,
            parent=parent,
            uri_prop='object-uri',
            name_prop='name',
            query_props=[])
        self._list_items = []

    def list(self, full_properties=False, filter_args=None):
        # We need to have a quick way to mock this method. This mocked
        # implementation basically uses the _list_items instance variable,
        # and then applies the client-side filtering on top of it.
        result_list = []
        for obj in self._list_items:
            if not filter_args or self._matches_filters(obj, filter_args):
                result_list.append(obj)
        return result_list


class ManagerTests(unittest.TestCase):
    def setUp(self):
        self.manager = MyManager()

        self.resource = MyResource(self.manager, "foo-uri",
                                   properties={"name": "foo-name",
                                               "other": "foo-other"})
        self.manager._list_items = [self.resource]

    def test_findall_attribute(self):
        items = self.manager.findall(other="foo-other")
        self.assertIn(self.resource, items)

    def test_findall_attribute_no_result(self):
        items = self.manager.findall(other="not-exists")
        self.assertEqual([], items)

    def test_findall_name(self):
        items = self.manager.findall(name="foo-name")
        self.assertEqual(1, len(items))
        item = items[0]
        self.assertEqual("foo-name", item.properties['name'])
        self.assertEqual("foo-uri", item.properties['object-uri'])

    def test_findall_name_no_result(self):
        items = self.manager.findall(name="not-exists")
        self.assertEqual([], items)
